//// [tests/cases/compiler/breakInIterationOrSwitchStatement3.ts] ////

//// [breakInIterationOrSwitchStatement3.ts]
for (;;) {
  break;
}

//// [breakInIterationOrSwitchStatement3.js]
"use strict";
for (;;) {
    break;
}
