//// [tests/cases/conformance/es6/variableDeclarations/VariableDeclaration5_es6.ts] ////

//// [VariableDeclaration5_es6.ts]
const a: number = 1

//// [VariableDeclaration5_es6.js]
"use strict";
const a = 1;
