//// [tests/cases/conformance/es6/yieldExpressions/YieldExpression13_es6.ts] ////

//// [YieldExpression13_es6.ts]
function* foo() { yield }

//// [YieldExpression13_es6.js]
"use strict";
function* foo() { yield; }
