//// [tests/cases/conformance/es6/computedProperties/computedPropertyNamesContextualType6_ES5.ts] ////

//// [computedPropertyNamesContextualType6_ES5.ts]
interface I<T> {
    [s: string]: T;
}

declare function foo<T>(obj: I<T>): T

foo({
    p: "",
    0: () => { },
    ["hi" + "bye"]: true,
    [0 + 1]: 0,
    [+"hi"]: [0]
});

//// [computedPropertyNamesContextualType6_ES5.js]
"use strict";
foo({
    p: "",
    0: () => { },
    ["hi" + "bye"]: true,
    [0 + 1]: 0,
    [+"hi"]: [0]
});
