//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of2.ts] ////

//// [ES5For-of2.ts]
for (var v of []) {
    var x = v;
}

//// [ES5For-of2.js]
"use strict";
for (var v of []) {
    var x = v;
}
