//// [tests/cases/conformance/async/es6/asyncArrowFunction/asyncArrowFunction2_es6.ts] ////

//// [asyncArrowFunction2_es6.ts]
var f = (await) => {
}

//// [asyncArrowFunction2_es6.js]
"use strict";
var f = (await) => {
};
