//// [tests/cases/conformance/es6/destructuring/destructuringVoidStrictNullChecks.ts] ////

//// [destructuringVoidStrictNullChecks.ts]
declare const v: void;
const {} = v;


//// [destructuringVoidStrictNullChecks.js]
"use strict";
const {} = v;
