//// [tests/cases/conformance/async/es6/asyncArrowFunction/asyncArrowFunction4_es6.ts] ////

//// [asyncArrowFunction4_es6.ts]
var await = () => {
}

//// [asyncArrowFunction4_es6.js]
"use strict";
var await = () => {
};
