//// [tests/cases/conformance/externalModules/es6/es6modulekindWithES5Target4.ts] ////

//// [es6modulekindWithES5Target4.ts]
class E { }
export default E;

//// [es6modulekindWithES5Target4.js]
class E {
}
export default E;
