//// [tests/cases/conformance/es6/yieldExpressions/YieldExpression18_es6.ts] ////

//// [YieldExpression18_es6.ts]
"use strict";
yield(foo);

//// [YieldExpression18_es6.js]
"use strict";
yield(foo);
