//// [tests/cases/conformance/types/primitives/undefined/directReferenceToUndefined.ts] ////

//// [directReferenceToUndefined.ts]
var x: Undefined;
var y = undefined;

//// [directReferenceToUndefined.js]
"use strict";
var x;
var y = undefined;
