//// [tests/cases/conformance/externalModules/es6/es6modulekindExportClassNameWithObject.ts] ////

//// [es6modulekindExportClassNameWithObject.ts]
export class Object {}


//// [es6modulekindExportClassNameWithObject.js]
export class Object {
}
