//// [tests/cases/conformance/async/es2017/functionDeclarations/asyncFunctionDeclaration12_es2017.ts] ////

//// [asyncFunctionDeclaration12_es2017.ts]
var v = async function await(): Promise<void> { }

//// [asyncFunctionDeclaration12_es2017.js]
"use strict";
var v = async function await() { };
