//// [tests/cases/compiler/collisionExportsRequireAndAmbientEnum.ts] ////

//// [collisionExportsRequireAndAmbientEnum_externalmodule.ts]
export declare enum require {
    _thisVal1,
    _thisVal2,
}
export declare enum exports {
    _thisVal1,
    _thisVal2,
}
declare namespace m1 {
    enum require {
        _thisVal1,
        _thisVal2,
    }
    enum exports {
        _thisVal1,
        _thisVal2,
    }
}
namespace m2 {
    export declare enum require { 
        _thisVal1,
        _thisVal2,
    }
    export declare enum exports {
        _thisVal1,
        _thisVal2,
    }
}

//// [collisionExportsRequireAndAmbientEnum_globalFile.ts]
declare enum require {
    _thisVal1,
    _thisVal2,
}
declare enum exports {
    _thisVal1,
    _thisVal2,
}
declare namespace m3 {
    enum require {
        _thisVal1,
        _thisVal2,
    }
    enum exports {
        _thisVal1,
        _thisVal2,
    }
}
namespace m4 {
    export declare enum require {
        _thisVal1,
        _thisVal2,
    }
    export declare enum exports {
        _thisVal1,
        _thisVal2,
    }
}

//// [collisionExportsRequireAndAmbientEnum_externalmodule.js]
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var m2;
(function (m2) {
})(m2 || (m2 = {}));
//// [collisionExportsRequireAndAmbientEnum_globalFile.js]
"use strict";
var m4;
(function (m4) {
})(m4 || (m4 = {}));
