//// [tests/cases/compiler/commentOnArrayElement16.ts] ////

//// [commentOnArrayElement16.ts]
const array = [
    // comment start
    1,
    2,
    // comment end
];


//// [commentOnArrayElement16.js]
"use strict";
const array = [
    // comment start
    1,
    2,
    // comment end
];
