//// [tests/cases/compiler/contextualTypingFunctionReturningFunction2.ts] ////

//// [contextualTypingFunctionReturningFunction2.ts]
declare function f(n: number): void;
declare function f(cb: () => (n: number) => number): void;

f(() => n => n);


//// [contextualTypingFunctionReturningFunction2.js]
"use strict";
f(() => n => n);
