//// [tests/cases/compiler/commentOnArrayElement4.ts] ////

//// [commentOnArrayElement4.ts]
const array = [
    /* element 1 */
    1,
    /* end of element 1 */
];


//// [commentOnArrayElement4.js]
"use strict";
const array = [
    /* element 1 */
    1,
    /* end of element 1 */
];
