//// [tests/cases/conformance/parser/ecmascript5/ErrorRecovery/ArrowFunctions/ArrowFunction4.ts] ////

//// [ArrowFunction4.ts]
var v = (a, b) => {
   
};

//// [ArrowFunction4.js]
"use strict";
var v = (a, b) => {
};
