//// [tests/cases/compiler/continueInIterationStatement1.ts] ////

//// [continueInIterationStatement1.ts]
while (true) {
  continue;
}

//// [continueInIterationStatement1.js]
"use strict";
while (true) {
    continue;
}
