//// [tests/cases/compiler/doNotEmitPinnedCommentNotOnTopOfFile.ts] ////

//// [doNotEmitPinnedCommentNotOnTopOfFile.ts]
var x = 10;

/*!

    multi line
    comment
*/

var x = 10;

//// [doNotEmitPinnedCommentNotOnTopOfFile.js]
"use strict";
var x = 10;
var x = 10;
