//// [tests/cases/conformance/es6/functionDeclarations/FunctionDeclaration1_es6.ts] ////

//// [FunctionDeclaration1_es6.ts]
function * foo() {
}

//// [FunctionDeclaration1_es6.js]
"use strict";
function* foo() {
}
