//// [tests/cases/conformance/es6/functionDeclarations/FunctionDeclaration6_es6.ts] ////

//// [FunctionDeclaration6_es6.ts]
function*foo(a = yield) {
}

//// [FunctionDeclaration6_es6.js]
"use strict";
function* foo(a = yield) {
}
