//// [tests/cases/compiler/commentOnDecoratedClassDeclaration.ts] ////

//// [commentOnDecoratedClassDeclaration.ts]
declare function decorator(x: string): any;

/**
 * Leading trivia
 */
@decorator("hello")
class Remote { }

/**
 * Floating Comment
 */

@decorator("hi")
class AnotherRomote {
    constructor() {}
}

//// [commentOnDecoratedClassDeclaration.js]
"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/**
 * Leading trivia
 */
let Remote = class Remote {
};
Remote = __decorate([
    decorator("hello")
], Remote);
/**
 * Floating Comment
 */
let AnotherRomote = class AnotherRomote {
    constructor() { }
};
AnotherRomote = __decorate([
    decorator("hi")
], AnotherRomote);
