//// [tests/cases/conformance/es6/arrowFunction/emitArrowFunctionWhenUsingArguments03_ES6.ts] ////

//// [emitArrowFunctionWhenUsingArguments03_ES6.ts]
var arguments;
var a = () => arguments;

//// [emitArrowFunctionWhenUsingArguments03_ES6.js]
"use strict";
var arguments;
var a = () => arguments;
