//// [tests/cases/compiler/commentOnArrayElement14.ts] ////

//// [commentOnArrayElement14.ts]
const array = [1 /* comment */];


//// [commentOnArrayElement14.js]
"use strict";
const array = [1 /* comment */];
