//// [tests/cases/compiler/continueNotInIterationStatement4.ts] ////

//// [continueNotInIterationStatement4.ts]
TWO:
while (true){
  var x = () => {
    continue TWO;
  }
}


//// [continueNotInIterationStatement4.js]
"use strict";
TWO: while (true) {
    var x = () => {
        continue TWO;
    };
}
