//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of29.ts] ////

//// [ES5For-of29.ts]
for (const {x: a = 0, y: b = 1} of [2, 3]) {
    a;
    b;
}

//// [ES5For-of29.js]
"use strict";
for (const { x: a = 0, y: b = 1 } of [2, 3]) {
    a;
    b;
}
