//// [tests/cases/compiler/convertClassExpressionToFunctionFromObjectProperty1.ts] ////

//// [convertClassExpressionToFunctionFromObjectProperty1.ts]
const foo: any = {};

// properties
foo.x = class {
    constructor () {}
}
foo.y = class {
    constructor () {}
}

// keywords
foo.break = class {
    constructor () {}
}
foo.case = class {
    constructor () {}
}
foo.catch = class {
    constructor () {}
}
foo.class = class {
    constructor () {}
}
foo.const = class {
    constructor () {}
}
foo.continue = class {
    constructor () {}
}
foo.debugger = class {
    constructor () {}
}
foo.default = class {
    constructor () {}
}
foo.delete = class {
    constructor () {}
}
foo.do = class {
    constructor () {}
}
foo.else = class {
    constructor () {}
}
foo.enum = class {
    constructor () {}
}
foo.export = class {
    constructor () {}
}
foo.extends = class {
    constructor () {}
}
foo.false = class {
    constructor () {}
}
foo.finally = class {
    constructor () {}
}
foo.for = class {
    constructor () {}
}
foo.function = class {
    constructor () {}
}
foo.if = class {
    constructor () {}
}
foo.import = class {
    constructor () {}
}
foo.in = class {
    constructor () {}
}
foo.instanceof = class {
    constructor () {}
}
foo.new = class {
    constructor () {}
}
foo.null = class {
    constructor () {}
}
foo.return = class {
    constructor () {}
}
foo.super = class {
    constructor () {}
}
foo.switch = class {
    constructor () {}
}
foo.this = class {
    constructor () {}
}
foo.throw = class {
    constructor () {}
}
foo.true = class {
    constructor () {}
}
foo.try = class {
    constructor () {}
}
foo.typeof = class {
    constructor () {}
}
foo.var = class {
    constructor () {}
}
foo.void = class {
    constructor () {}
}
foo.while = class {
    constructor () {}
}
foo.with = class {
    constructor () {}
}
foo.implements = class {
    constructor () {}
}
foo.interface = class {
    constructor () {}
}
foo.let = class {
    constructor () {}
}
foo.package = class {
    constructor () {}
}
foo.private = class {
    constructor () {}
}
foo.protected = class {
    constructor () {}
}
foo.public = class {
    constructor () {}
}
foo.static = class {
    constructor () {}
}
foo.yield = class {
    constructor () {}
}
foo.abstract = class {
    constructor () {}
}
foo.as = class {
    constructor () {}
}
foo.asserts = class {
    constructor () {}
}
foo.any = class {
    constructor () {}
}
foo.async = class {
    constructor () {}
}
foo.await = class {
    constructor () {}
}
foo.boolean = class {
    constructor () {}
}
foo.constructor = class {
    constructor () {}
}
foo.declare = class {
    constructor () {}
}
foo.get = class {
    constructor () {}
}
foo.infer = class {
    constructor () {}
}
foo.is = class {
    constructor () {}
}
foo.keyof = class {
    constructor () {}
}
foo.module = class {
    constructor () {}
}
foo.namespace = class {
    constructor () {}
}
foo.never = class {
    constructor () {}
}
foo.readonly = class {
    constructor () {}
}
foo.require = class {
    constructor () {}
}
foo.number = class {
    constructor () {}
}
foo.object = class {
    constructor () {}
}
foo.set = class {
    constructor () {}
}
foo.string = class {
    constructor () {}
}
foo.symbol = class {
    constructor () {}
}
foo.type = class {
    constructor () {}
}
foo.undefined = class {
    constructor () {}
}
foo.unique = class {
    constructor () {}
}
foo.unknown = class {
    constructor () {}
}
foo.from = class {
    constructor () {}
}
foo.global = class {
    constructor () {}
}
foo.bigint = class {
    constructor () {}
}
foo.of = class {
    constructor () {}
}


//// [convertClassExpressionToFunctionFromObjectProperty1.js]
"use strict";
const foo = {};
// properties
foo.x = class {
    constructor() { }
};
foo.y = class {
    constructor() { }
};
// keywords
foo.break = class {
    constructor() { }
};
foo.case = class {
    constructor() { }
};
foo.catch = class {
    constructor() { }
};
foo.class = class {
    constructor() { }
};
foo.const = class {
    constructor() { }
};
foo.continue = class {
    constructor() { }
};
foo.debugger = class {
    constructor() { }
};
foo.default = class {
    constructor() { }
};
foo.delete = class {
    constructor() { }
};
foo.do = class {
    constructor() { }
};
foo.else = class {
    constructor() { }
};
foo.enum = class {
    constructor() { }
};
foo.export = class {
    constructor() { }
};
foo.extends = class {
    constructor() { }
};
foo.false = class {
    constructor() { }
};
foo.finally = class {
    constructor() { }
};
foo.for = class {
    constructor() { }
};
foo.function = class {
    constructor() { }
};
foo.if = class {
    constructor() { }
};
foo.import = class {
    constructor() { }
};
foo.in = class {
    constructor() { }
};
foo.instanceof = class {
    constructor() { }
};
foo.new = class {
    constructor() { }
};
foo.null = class {
    constructor() { }
};
foo.return = class {
    constructor() { }
};
foo.super = class {
    constructor() { }
};
foo.switch = class {
    constructor() { }
};
foo.this = class {
    constructor() { }
};
foo.throw = class {
    constructor() { }
};
foo.true = class {
    constructor() { }
};
foo.try = class {
    constructor() { }
};
foo.typeof = class {
    constructor() { }
};
foo.var = class {
    constructor() { }
};
foo.void = class {
    constructor() { }
};
foo.while = class {
    constructor() { }
};
foo.with = class {
    constructor() { }
};
foo.implements = class {
    constructor() { }
};
foo.interface = class {
    constructor() { }
};
foo.let = class {
    constructor() { }
};
foo.package = class {
    constructor() { }
};
foo.private = class {
    constructor() { }
};
foo.protected = class {
    constructor() { }
};
foo.public = class {
    constructor() { }
};
foo.static = class {
    constructor() { }
};
foo.yield = class {
    constructor() { }
};
foo.abstract = class {
    constructor() { }
};
foo.as = class {
    constructor() { }
};
foo.asserts = class {
    constructor() { }
};
foo.any = class {
    constructor() { }
};
foo.async = class {
    constructor() { }
};
foo.await = class {
    constructor() { }
};
foo.boolean = class {
    constructor() { }
};
foo.constructor = class {
    constructor() { }
};
foo.declare = class {
    constructor() { }
};
foo.get = class {
    constructor() { }
};
foo.infer = class {
    constructor() { }
};
foo.is = class {
    constructor() { }
};
foo.keyof = class {
    constructor() { }
};
foo.module = class {
    constructor() { }
};
foo.namespace = class {
    constructor() { }
};
foo.never = class {
    constructor() { }
};
foo.readonly = class {
    constructor() { }
};
foo.require = class {
    constructor() { }
};
foo.number = class {
    constructor() { }
};
foo.object = class {
    constructor() { }
};
foo.set = class {
    constructor() { }
};
foo.string = class {
    constructor() { }
};
foo.symbol = class {
    constructor() { }
};
foo.type = class {
    constructor() { }
};
foo.undefined = class {
    constructor() { }
};
foo.unique = class {
    constructor() { }
};
foo.unknown = class {
    constructor() { }
};
foo.from = class {
    constructor() { }
};
foo.global = class {
    constructor() { }
};
foo.bigint = class {
    constructor() { }
};
foo.of = class {
    constructor() { }
};
