//// [tests/cases/compiler/differentTypesWithSameName.ts] ////

//// [differentTypesWithSameName.ts]
namespace m {
  export class variable{
    s: string;
  }
  export function doSomething(v: m.variable) {
    
  }
}

class variable {
 t: number;
}


var v: variable = new variable();
m.doSomething(v);

//// [differentTypesWithSameName.js]
"use strict";
var m;
(function (m) {
    class variable {
    }
    m.variable = variable;
    function doSomething(v) {
    }
    m.doSomething = doSomething;
})(m || (m = {}));
class variable {
}
var v = new variable();
m.doSomething(v);
