//// [tests/cases/conformance/es6/memberFunctionDeclarations/MemberFunctionDeclaration1_es6.ts] ////

//// [MemberFunctionDeclaration1_es6.ts]
class C {
   *foo() { }
}

//// [MemberFunctionDeclaration1_es6.js]
"use strict";
class C {
    *foo() { }
}
