//// [tests/cases/conformance/statements/for-ofStatements/ES5For-ofTypeCheck2.ts] ////

//// [ES5For-ofTypeCheck2.ts]
for (var v of [true]) { }

//// [ES5For-ofTypeCheck2.js]
"use strict";
for (var v of [true]) { }
