//// [tests/cases/conformance/es6/yieldExpressions/YieldExpression9_es6.ts] ////

//// [YieldExpression9_es6.ts]
var v = function*() {
  yield(foo);
}

//// [YieldExpression9_es6.js]
"use strict";
var v = function* () {
    yield (foo);
};
