//// [tests/cases/conformance/expressions/binaryOperators/arithmeticOperator/arithmeticOperatorWithOnlyNullValueOrUndefinedValue.ts] ////

//// [arithmeticOperatorWithOnlyNullValueOrUndefinedValue.ts]
// operator *
var ra1 = null * null;
var ra2 = null * undefined;
var ra3 = undefined * null;
var ra4 = undefined * undefined;

// operator /
var rb1 = null / null;
var rb2 = null / undefined;
var rb3 = undefined / null;
var rb4 = undefined / undefined;

// operator %
var rc1 = null % null;
var rc2 = null % undefined;
var rc3 = undefined % null;
var rc4 = undefined % undefined;

// operator -
var rd1 = null - null;
var rd2 = null - undefined;
var rd3 = undefined - null;
var rd4 = undefined - undefined;

// operator <<
var re1 = null << null;
var re2 = null << undefined;
var re3 = undefined << null;
var re4 = undefined << undefined;

// operator >>
var rf1 = null >> null;
var rf2 = null >> undefined;
var rf3 = undefined >> null;
var rf4 = undefined >> undefined;

// operator >>>
var rg1 = null >>> null;
var rg2 = null >>> undefined;
var rg3 = undefined >>> null;
var rg4 = undefined >>> undefined;

// operator &
var rh1 = null & null;
var rh2 = null & undefined;
var rh3 = undefined & null;
var rh4 = undefined & undefined;

// operator ^
var ri1 = null ^ null;
var ri2 = null ^ undefined;
var ri3 = undefined ^ null;
var ri4 = undefined ^ undefined;

// operator |
var rj1 = null | null;
var rj2 = null | undefined;
var rj3 = undefined | null;
var rj4 = undefined | undefined;

//// [arithmeticOperatorWithOnlyNullValueOrUndefinedValue.js]
"use strict";
// operator *
var ra1 = null * null;
var ra2 = null * undefined;
var ra3 = undefined * null;
var ra4 = undefined * undefined;
// operator /
var rb1 = null / null;
var rb2 = null / undefined;
var rb3 = undefined / null;
var rb4 = undefined / undefined;
// operator %
var rc1 = null % null;
var rc2 = null % undefined;
var rc3 = undefined % null;
var rc4 = undefined % undefined;
// operator -
var rd1 = null - null;
var rd2 = null - undefined;
var rd3 = undefined - null;
var rd4 = undefined - undefined;
// operator <<
var re1 = null << null;
var re2 = null << undefined;
var re3 = undefined << null;
var re4 = undefined << undefined;
// operator >>
var rf1 = null >> null;
var rf2 = null >> undefined;
var rf3 = undefined >> null;
var rf4 = undefined >> undefined;
// operator >>>
var rg1 = null >>> null;
var rg2 = null >>> undefined;
var rg3 = undefined >>> null;
var rg4 = undefined >>> undefined;
// operator &
var rh1 = null & null;
var rh2 = null & undefined;
var rh3 = undefined & null;
var rh4 = undefined & undefined;
// operator ^
var ri1 = null ^ null;
var ri2 = null ^ undefined;
var ri3 = undefined ^ null;
var ri4 = undefined ^ undefined;
// operator |
var rj1 = null | null;
var rj2 = null | undefined;
var rj3 = undefined | null;
var rj4 = undefined | undefined;
