Fs::
//// [/apath/a.ts]


//// [/apath/tsconfig.json]
{
                "files": [],
                "references": [{ "path": "/apath" }]
            }


configFileName:: /apath/tsconfig.json
FileNames::

Errors::

