/* Generated file to emulate the Harness.Parallel namespace. */

export * from "../parallel/shared.js";
import * as Host from "./Harness.Parallel.Host.js";
export { Host };
import * as Worker from "./Harness.Parallel.Worker.js";
export { Worker };
