//// [tests/cases/conformance/internalModules/DeclarationMerging/ModuleAndClassWithSameNameAndCommonRoot.ts] ////

//// [module.ts]
namespace X.Y {
    export namespace Point {
        export var Origin = new Point(0, 0);
    }
}

//// [classPoint.ts]
namespace X.Y {
    // duplicate identifier
    export class Point {
        constructor(x: number, y: number) {
            this.x = x;
            this.y = y;
        }
        x: number;
        y: number;
    }
}

//// [simple.ts]
namespace A {
    export var Instance = new A();
}

// duplicate identifier
class A {
    id: string;
}


//// [module.js]
"use strict";
var X;
(function (X) {
    var Y;
    (function (Y) {
        let Point;
        (function (Point) {
            Point.Origin = new Point(0, 0);
        })(Point = Y.Point || (Y.Point = {}));
    })(Y = X.Y || (X.Y = {}));
})(X || (X = {}));
//// [classPoint.js]
"use strict";
var X;
(function (X) {
    var Y;
    (function (Y) {
        // duplicate identifier
        class Point {
            constructor(x, y) {
                this.x = x;
                this.y = y;
            }
        }
        Y.Point = Point;
    })(Y = X.Y || (X.Y = {}));
})(X || (X = {}));
//// [simple.js]
"use strict";
var A;
(function (A) {
    A.Instance = new A();
})(A || (A = {}));
// duplicate identifier
class A {
}
