//// [tests/cases/conformance/es6/functionPropertyAssignments/FunctionPropertyAssignments1_es6.ts] ////

//// [FunctionPropertyAssignments1_es6.ts]
var v = { *foo() { } }

//// [FunctionPropertyAssignments1_es6.js]
"use strict";
var v = { *foo() { } };
