//// [tests/cases/conformance/es6/functionExpressions/FunctionExpression1_es6.ts] ////

//// [FunctionExpression1_es6.ts]
var v = function * () { }

//// [FunctionExpression1_es6.js]
"use strict";
var v = function* () { };
