//// [tests/cases/conformance/async/es5/asyncAliasReturnType_es5.ts] ////

//// [asyncAliasReturnType_es5.ts]
type PromiseAlias<T> = Promise<T>;

async function f(): PromiseAlias<void> {
}

//// [asyncAliasReturnType_es5.js]
"use strict";
function f() {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/];
        });
    });
}
