//// [tests/cases/compiler/commentOnArrayElement11.ts] ////

//// [commentOnArrayElement11.ts]
const array = [
    , /* comment */
];


//// [commentOnArrayElement11.js]
"use strict";
const array = [
    , /* comment */
];
