//// [tests/cases/compiler/classCannotExtendVar.ts] ////

//// [classCannotExtendVar.ts]
var Markup;

class Markup {
    constructor() {
    }
}


//// [classCannotExtendVar.js]
"use strict";
var Markup;
class Markup {
    constructor() {
    }
}
