//// [tests/cases/conformance/ambient/ambientDeclarationsPatterns_tooManyAsterisks.ts] ////

//// [ambientDeclarationsPatterns_tooManyAsterisks.ts]
declare module "too*many*asterisks" { }


//// [ambientDeclarationsPatterns_tooManyAsterisks.js]
"use strict";
