//// [tests/cases/compiler/asiInES6Classes.ts] ////

//// [asiInES6Classes.ts]
class Foo {

 

    defaults = {

        done: false

    }

 

    bar() {

        return 3;

    }

 

}


//// [asiInES6Classes.js]
"use strict";
class Foo {
    constructor() {
        this.defaults = {
            done: false
        };
    }
    bar() {
        return 3;
    }
}
