//// [tests/cases/conformance/async/es5/asyncArrowFunction/asyncArrowFunction11_es5.ts] ////

//// [asyncArrowFunction11_es5.ts]
// https://github.com/Microsoft/TypeScript/issues/24722
class A {
    b = async (...args: any[]) => {
        await Promise.resolve();
        const obj = { ["a"]: () => this }; // computed property name after `await` triggers case
    };
}

//// [asyncArrowFunction11_es5.js]
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// https://github.com/Microsoft/TypeScript/issues/24722
class A {
    constructor() {
        this.b = (...args) => __awaiter(this, void 0, void 0, function* () {
            yield Promise.resolve();
            const obj = { ["a"]: () => this }; // computed property name after `await` triggers case
        });
    }
}
