//// [tests/cases/compiler/es5-system2.ts] ////

//// [es5-system2.ts]
export var __esModule = 1;

//// [es5-system2.js]
System.register([], function (exports_1, context_1) {
    "use strict";
    var __esModule;
    var __moduleName = context_1 && context_1.id;
    return {
        setters: [],
        execute: function () {
            exports_1("__esModule", __esModule = 1);
        }
    };
});
