--optionName node 0.ts
CompilerOptions::
{}
WatchOptions::

FileNames::
0.ts
Errors::
error TS6064: Option 'optionName' can only be specified in 'tsconfig.json' file or set to 'null' on command line.
