//// [tests/cases/conformance/es6/destructuring/destructuringTypeAssertionsES5_5.ts] ////

//// [destructuringTypeAssertionsES5_5.ts]
var { x } = <any>0;

//// [destructuringTypeAssertionsES5_5.js]
"use strict";
var x = 0..x;
