Fs::
//// [/apath/a.ts]


//// [/apath/tsconfig.json]
{
  "files": [
    {
      "compilerOptions": {
        "experimentalDecorators": true,
        "allowJs": true
      }
    }
  ]
}


configFileName:: /apath/tsconfig.json
FileNames::

Errors::
[91merror[0m[90m TS5024: [0mCompiler option 'files' requires a value of type string.

