//// [tests/cases/conformance/es6/functionDeclarations/FunctionDeclaration8_es6.ts] ////

//// [FunctionDeclaration8_es6.ts]
var v = { [yield]: foo }

//// [FunctionDeclaration8_es6.js]
"use strict";
var v = { [yield]: foo };
