//// [tests/cases/compiler/commentOnArrayElement15.ts] ////

//// [commentOnArrayElement15.ts]
const array = [/* comment */ 1 /* comment */];


//// [commentOnArrayElement15.js]
"use strict";
const array = [/* comment */ 1 /* comment */];
