//// [tests/cases/conformance/es6/memberFunctionDeclarations/MemberFunctionDeclaration3_es6.ts] ////

//// [MemberFunctionDeclaration3_es6.ts]
class C {
   *[foo]() { }
}

//// [MemberFunctionDeclaration3_es6.js]
"use strict";
class C {
    *[foo]() { }
}
