//// [tests/cases/conformance/async/es2017/functionDeclarations/asyncFunctionDeclaration3_es2017.ts] ////

//// [asyncFunctionDeclaration3_es2017.ts]
function f(await = await) {
}

//// [asyncFunctionDeclaration3_es2017.js]
"use strict";
function f(await = await) {
}
