// [source.js.map]
{"version":3,"file":"source.js","sourceRoot":"","sources":["source.ts","another.html"],"names":[],"mappings":";ACAA,OAAO,CDAW"}

// [source.js]
"use strict";
changed;
//# sourceMappingURL=source.js.map