//// [tests/cases/conformance/es6/functionDeclarations/FunctionDeclaration11_es6.ts] ////

//// [FunctionDeclaration11_es6.ts]
function * yield() {
}

//// [FunctionDeclaration11_es6.js]
"use strict";
function* yield() {
}
