//// [tests/cases/compiler/ambiguousOverloadResolution.ts] ////

//// [ambiguousOverloadResolution.ts]
class A { }
class B extends A { x: number; }

declare function f(p: A, q: B): number;
declare function f(p: B, q: A): string;

var x: B;
var t: number = f(x, x); // Not an error

//// [ambiguousOverloadResolution.js]
"use strict";
class A {
}
class B extends A {
}
var x;
var t = f(x, x); // Not an error
