//// [tests/cases/compiler/collisionCodeGenEnumWithEnumMemberConflict.ts] ////

//// [collisionCodeGenEnumWithEnumMemberConflict.ts]
enum Color {
    Color,
    Thing = Color
}

//// [collisionCodeGenEnumWithEnumMemberConflict.js]
"use strict";
var Color;
(function (Color) {
    Color[Color["Color"] = 0] = "Color";
    Color[Color["Thing"] = 0] = "Thing";
})(Color || (Color = {}));
