--declarations --allowTS
CompilerOptions::
{}
WatchOptions::

FileNames::

Errors::
error TS5025: Unknown compiler option '--declarations'. Did you mean 'declaration'?
error TS5025: Unknown compiler option '--allowTS'. Did you mean 'allowJs'?
