//// [tests/cases/compiler/commentOnArrayElement7.ts] ////

//// [commentOnArrayElement7.ts]
const array = [/* element 1 */ 1, /* end of element 1 */];


//// [commentOnArrayElement7.js]
"use strict";
const array = [/* element 1 */ 1, /* end of element 1 */];
