//// [tests/cases/conformance/async/es6/asyncSetter_es6.ts] ////

//// [asyncSetter_es6.ts]
class C {
  async set foo(value) {
  }
}

//// [asyncSetter_es6.js]
"use strict";
class C {
    set foo(value) {
    }
}
