Fs::
//// [/apath/a.ts]


//// [/apath/b.js]


//// [/apath/jsconfig.json]
{
  "typeAcquisition": {
    "enable": false
  }
}


configFileName:: jsconfig.json
TypeAcquisition::
{
  "enable": false,
  "include": [],
  "exclude": []
}
Errors::

