//// [tests/cases/conformance/es6/destructuring/destructuringInFunctionType.ts] ////

//// [destructuringInFunctionType.ts]
interface a { a }
interface b { b }
interface c { c }

type T1 = ([a, b, c]);
type F1 = ([a, b, c]) => void;

type T2 = ({ a });
type F2 = ({ a }) => void;

type T3 = ([{ a: b }, { b: a }]);
type F3 = ([{ a: b }, { b: a }]) => void;

type T4 = ([{ a: [b, c] }]);
type F4 = ([{ a: [b, c] }]) => void;

type C1 = new ([{ a: [b, c] }]) => void;

var v1 = ([a, b, c]) => "hello";
var v2: ([a, b, c]) => string;


//// [destructuringInFunctionType.js]
"use strict";
var v1 = ([a, b, c]) => "hello";
var v2;


//// [destructuringInFunctionType.d.ts]
interface a {
    a: any;
}
interface b {
    b: any;
}
interface c {
    c: any;
}
type T1 = ([a, b, c]);
type F1 = ([a, b, c]: [any, any, any]) => void;
type T2 = ({
    a: any;
});
type F2 = ({ a }: {
    a: any;
}) => void;
type T3 = ([{
    a: b;
}, {
    b: a;
}]);
type F3 = ([{ a: b }, { b: a }]: [{
    a: any;
}, {
    b: any;
}]) => void;
type T4 = ([{
    a: [b, c];
}]);
type F4 = ([{ a: [b, c] }]: [{
    a: [any, any];
}]) => void;
type C1 = new ([{ a: [b, c] }]: [{
    a: [any, any];
}]) => void;
declare var v1: ([a, b, c]: [any, any, any]) => string;
declare var v2: ([a, b, c]: [any, any, any]) => string;
