//// [tests/cases/conformance/classes/classDeclarations/classWithPredefinedTypesAsNames.ts] ////

//// [classWithPredefinedTypesAsNames.ts]
// classes cannot use predefined types as names

class any { }
class number { }
class boolean { }
class string { }

//// [classWithPredefinedTypesAsNames.js]
"use strict";
// classes cannot use predefined types as names
class any {
}
class number {
}
class boolean {
}
class string {
}
