//// [tests/cases/compiler/declFileForVarList.ts] ////

//// [declFileForVarList.ts]
var x, y, z = 1;
var x1 = 1, y2 = 2, z2 = 3;

//// [declFileForVarList.js]
"use strict";
var x, y, z = 1;
var x1 = 1, y2 = 2, z2 = 3;


//// [declFileForVarList.d.ts]
declare var x: any, y: any, z: number;
declare var x1: number, y2: number, z2: number;
