//// [tests/cases/compiler/continueInIterationStatement3.ts] ////

//// [continueInIterationStatement3.ts]
for (;;) {
  continue;
}

//// [continueInIterationStatement3.js]
"use strict";
for (;;) {
    continue;
}
