//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of33.ts] ////

//// [ES5For-of33.ts]
for (var v of ['a', 'b', 'c']) {
    console.log(v);
}

//// [ES5For-of33.js]
"use strict";
for (var v of ['a', 'b', 'c']) {
    console.log(v);
}
//# sourceMappingURL=ES5For-of33.js.map