0.ts --moduleResolution
CompilerOptions::
{}
WatchOptions::

FileNames::
0.ts
Errors::
error TS6044: Compiler option 'moduleResolution' expects an argument.
error TS6046: Argument for '--moduleResolution' option must be: 'node16', 'nodenext', 'bundler'.
