//// [tests/cases/conformance/es6/memberFunctionDeclarations/MemberFunctionDeclaration5_es6.ts] ////

//// [MemberFunctionDeclaration5_es6.ts]
class C {
   *
}

//// [MemberFunctionDeclaration5_es6.js]
"use strict";
class C {
}
