//// [tests/cases/compiler/ambientEnumElementInitializer6.ts] ////

//// [ambientEnumElementInitializer6.ts]
declare namespace M {
    enum E {
        e = 3
    }
}

//// [ambientEnumElementInitializer6.js]
"use strict";
