//// [tests/cases/conformance/es6/computedProperties/computedPropertyNames24_ES6.ts] ////

//// [computedPropertyNames24_ES6.ts]
class Base {
    bar() {
        return 0;
    }
}
class C extends Base {
    // Gets emitted as super, not _super, which is consistent with
    // use of super in static properties initializers.
    [super.bar()]() { }
}

//// [computedPropertyNames24_ES6.js]
"use strict";
class Base {
    bar() {
        return 0;
    }
}
class C extends Base {
    // Gets emitted as super, not _super, which is consistent with
    // use of super in static properties initializers.
    [super.bar()]() { }
}
