//// [tests/cases/conformance/classes/classDeclarations/classWithSemicolonClassElement1.ts] ////

//// [classWithSemicolonClassElement1.ts]
class C {
    ;
}

//// [classWithSemicolonClassElement1.js]
"use strict";
class C {
    ;
}
