//// [tests/cases/conformance/async/es5/asyncClass_es5.ts] ////

//// [asyncClass_es5.ts]
async class C {
}

//// [asyncClass_es5.js]
"use strict";
var C = /** @class */ (function () {
    function C() {
    }
    return C;
}());
