/* Generated file to emulate the ts.server namespace. */

export * from "../../jsTyping/_namespaces/ts.server.js";
import * as typingsInstaller from "./ts.server.typingsInstaller.js";
export { typingsInstaller };
