//// [tests/cases/conformance/async/es2017/functionDeclarations/asyncFunctionDeclaration2_es2017.ts] ////

//// [asyncFunctionDeclaration2_es2017.ts]
function f(await) {
}

//// [asyncFunctionDeclaration2_es2017.js]
"use strict";
function f(await) {
}
