//// [tests/cases/conformance/es6/classDeclaration/classWithSemicolonClassElementES61.ts] ////

//// [classWithSemicolonClassElementES61.ts]
class C {
    ;
}

//// [classWithSemicolonClassElementES61.js]
"use strict";
class C {
    ;
}
