//// [tests/cases/conformance/parser/ecmascript5/ErrorRecovery/ArrowFunctions/ArrowFunction3.ts] ////

//// [ArrowFunction3.ts]
var v = (a): => {
   
};

//// [ArrowFunction3.js]
"use strict";
var v = (a);
{
}
;
