//// [tests/cases/compiler/ArrowFunctionExpression1.ts] ////

//// [ArrowFunctionExpression1.ts]
var v = (public x: string) => { };

//// [ArrowFunctionExpression1.js]
"use strict";
var v = (x) => { };
