//// [tests/cases/compiler/defaultKeywordWithoutExport2.ts] ////

//// [defaultKeywordWithoutExport2.ts]
default function () {

}

//// [defaultKeywordWithoutExport2.js]
"use strict";
function () {
}
