//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of5.ts] ////

//// [ES5For-of5.ts]
for (var _a of []) {
    var x = _a;
}

//// [ES5For-of5.js]
"use strict";
for (var _a of []) {
    var x = _a;
}
