//// [tests/cases/compiler/es5-asyncFunctionForInStatements.ts] ////

//// [es5-asyncFunctionForInStatements.ts]
declare var x, y, z, a, b, c;

async function forInStatement0() {
    for (x in y) { z; }
}

async function forInStatement1() {
    for (x in await y) { z; }
}

async function forInStatement2() {
    for (x in y) { await z; }
}

async function forInStatement3() {
    for ((await x).a in y) { z; }
}

async function forInStatement4() {
    for (x.a in await y) { z; }
}

async function forInStatement5() {
    for (x.a in y) { await z; }
}

async function forInStatement6() {
    for (var a in y) { z; }
}

async function forInStatement7() {
    for (var b in await y) { z; }
}

async function forInStatement8() {
    for (var c in y) { await z; }
}

//// [es5-asyncFunctionForInStatements.js]
"use strict";
function forInStatement0() {
    return __awaiter(this, void 0, void 0, function* () {
        for (x in y) {
            z;
        }
    });
}
function forInStatement1() {
    return __awaiter(this, void 0, void 0, function* () {
        for (x in yield y) {
            z;
        }
    });
}
function forInStatement2() {
    return __awaiter(this, void 0, void 0, function* () {
        for (x in y) {
            yield z;
        }
    });
}
function forInStatement3() {
    return __awaiter(this, void 0, void 0, function* () {
        for ((yield x).a in y) {
            z;
        }
    });
}
function forInStatement4() {
    return __awaiter(this, void 0, void 0, function* () {
        for (x.a in yield y) {
            z;
        }
    });
}
function forInStatement5() {
    return __awaiter(this, void 0, void 0, function* () {
        for (x.a in y) {
            yield z;
        }
    });
}
function forInStatement6() {
    return __awaiter(this, void 0, void 0, function* () {
        for (var a in y) {
            z;
        }
    });
}
function forInStatement7() {
    return __awaiter(this, void 0, void 0, function* () {
        for (var b in yield y) {
            z;
        }
    });
}
function forInStatement8() {
    return __awaiter(this, void 0, void 0, function* () {
        for (var c in y) {
            yield z;
        }
    });
}
