--lib es5,es2015.symbol.wellknown --target es5 0.ts
CompilerOptions::
{
  "lib": [
    "lib.es5.d.ts",
    "lib.es2015.symbol.wellknown.d.ts"
  ],
  "target": 1
}
WatchOptions::

FileNames::
0.ts
Errors::
