//// [tests/cases/conformance/parser/ecmascript5/Protected/Protected9.ts] ////

//// [Protected9.ts]
class C {
   constructor(protected p) { }
}

//// [Protected9.js]
"use strict";
class C {
    constructor(p) {
        this.p = p;
    }
}
