//// [tests/cases/conformance/es6/destructuring/arrayAssignmentPatternWithAny.ts] ////

//// [arrayAssignmentPatternWithAny.ts]
var a: any;
var x: string;
[x] = a;

//// [arrayAssignmentPatternWithAny.js]
"use strict";
var a;
var x;
[x] = a;
