//// [tests/cases/conformance/es6/variableDeclarations/VariableDeclaration11_es6.ts] ////

//// [VariableDeclaration11_es6.ts]
"use strict";
let

//// [VariableDeclaration11_es6.js]
"use strict";
let;
