//// [tests/cases/compiler/FunctionDeclaration6.ts] ////

//// [FunctionDeclaration6.ts]
{
    function foo();
    function bar() { }
}

//// [FunctionDeclaration6.js]
"use strict";
{
    function bar() { }
}
