//// [tests/cases/conformance/async/es5/functionDeclarations/asyncFunctionDeclaration8_es5.ts] ////

//// [asyncFunctionDeclaration8_es5.ts]
var v = { [await]: foo }

//// [asyncFunctionDeclaration8_es5.js]
"use strict";
var v = { [await]: foo };
