//// [tests/cases/compiler/classMethodWithKeywordName1.ts] ////

//// [classMethodWithKeywordName1.ts]
class C {
 static try() {}
}

//// [classMethodWithKeywordName1.js]
"use strict";
class C {
    static try() { }
}
