//// [tests/cases/conformance/es6/computedProperties/computedPropertyNames29_ES6.ts] ////

//// [computedPropertyNames29_ES6.ts]
class C {
    bar() {
        () => {
            var obj = {
                [this.bar()]() { } // needs capture
            };
        }
        return 0;
    }
}

//// [computedPropertyNames29_ES6.js]
"use strict";
class C {
    bar() {
        () => {
            var obj = {
                [this.bar()]() { } // needs capture
            };
        };
        return 0;
    }
}
