//// [tests/cases/conformance/salsa/constructorNameInGenerator.ts] ////

//// [constructorNameInGenerator.ts]
class C2 {
    *constructor() {}
}


//// [constructorNameInGenerator.js]
"use strict";
class C2 {
    *constructor() { }
}
