//// [tests/cases/conformance/es6/yieldExpressions/YieldExpression15_es6.ts] ////

//// [YieldExpression15_es6.ts]
var v = () => {
     yield foo
  }

//// [YieldExpression15_es6.js]
"use strict";
var v = () => {
    yield foo;
};
