//// [tests/cases/conformance/async/es2017/asyncArrowFunction/asyncArrowFunction1_es2017.ts] ////

//// [asyncArrowFunction1_es2017.ts]
var foo = async (): Promise<void> => {
};

//// [asyncArrowFunction1_es2017.js]
"use strict";
var foo = async () => {
};
