//// [tests/cases/conformance/statements/VariableStatements/usingDeclarations/awaitUsingDeclarations.11.ts] ////

//// [awaitUsingDeclarations.11.ts]
export await using x = null;
declare await using y: null;


//// [awaitUsingDeclarations.11.js]
export await using x = null;
