Fs::
//// [/apath/a.ts]


//// [/apath/tsconfig.json]
{
  "compilerOptions": {
    "help": true
  }
}


configFileName:: /apath/tsconfig.json
FileNames::
/apath/a.ts
Errors::
[96mtsconfig.json[0m:[93m3[0m:[93m5[0m - [91merror[0m[90m TS6266: [0mOption 'help' can only be specified on command line.

[7m3[0m     "help": true
[7m [0m [91m    ~~~~~~[0m

