//// [tests/cases/conformance/classes/classStaticBlock/classStaticBlock14.ts] ////

//// [classStaticBlock14.ts]
class C {
  static #_1 = 1;
  static #_3 = 1;
  static #_5 = 1;

  static {}
  static {}
  static {}
  static {}
  static {}
  static {}
}


//// [classStaticBlock14.js]
"use strict";
var _a, _C__1, _C__3, _C__5;
class C {
}
_a = C;
_C__1 = { value: 1 };
_C__3 = { value: 1 };
_C__5 = { value: 1 };
(() => {
})();
(() => {
})();
(() => {
})();
(() => {
})();
(() => {
})();
(() => {
})();
