//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of21.ts] ////

//// [ES5For-of21.ts]
for (let v of []) {
    for (let _i of []) { }
}

//// [ES5For-of21.js]
"use strict";
for (let v of []) {
    for (let _i of []) { }
}
