//// [tests/cases/compiler/es5-commonjs2.ts] ////

//// [es5-commonjs2.ts]
export default 1;


//// [es5-commonjs2.js]
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = 1;
