//// [tests/cases/conformance/statements/for-ofStatements/ES5For-ofTypeCheck1.ts] ////

//// [ES5For-ofTypeCheck1.ts]
for (var v of "") { }

//// [ES5For-ofTypeCheck1.js]
"use strict";
for (var v of "") { }
