//// [tests/cases/conformance/async/es2017/functionDeclarations/asyncFunctionDeclaration11_es2017.ts] ////

//// [asyncFunctionDeclaration11_es2017.ts]
async function await(): Promise<void> {
}

//// [asyncFunctionDeclaration11_es2017.js]
"use strict";
async function await() {
}
