//// [tests/cases/conformance/async/es2017/asyncArrowFunction/arrowFunctionWithParameterNameAsync_es2017.ts] ////

//// [arrowFunctionWithParameterNameAsync_es2017.ts]
const x = async => async;

//// [arrowFunctionWithParameterNameAsync_es2017.js]
"use strict";
const x = async => async;
