//// [tests/cases/compiler/conditionalExpressionNewLine6.ts] ////

//// [conditionalExpressionNewLine6.ts]
var v = a 
  ? b
  : c;

//// [conditionalExpressionNewLine6.js]
"use strict";
var v = a
    ? b
    : c;
