//// [tests/cases/conformance/es6/destructuring/emptyVariableDeclarationBindingPatterns02_ES5.ts] ////

//// [emptyVariableDeclarationBindingPatterns02_ES5.ts]
(function () {
    var {};
    let {};
    const {};

    var [];
    let [];
    const [];
})();

//// [emptyVariableDeclarationBindingPatterns02_ES5.js]
"use strict";
(function () {
    var _a = void 0;
    var _b = void 0;
    var _c = void 0;
    var _d = void 0;
    var _e = void 0;
    var _f = void 0;
})();


//// [emptyVariableDeclarationBindingPatterns02_ES5.d.ts]
