//// [tests/cases/compiler/es6ExportDefaultFunctionDeclaration2.ts] ////

//// [es6ExportDefaultFunctionDeclaration2.ts]
export default function () { }


//// [es6ExportDefaultFunctionDeclaration2.js]
export default function () { }


//// [es6ExportDefaultFunctionDeclaration2.d.ts]
export default function (): void;
