//// [tests/cases/compiler/commentsPropertySignature1.ts] ////

//// [commentsPropertySignature1.ts]
var a = {
    /** own x*/
    x: 0
};


//// [commentsPropertySignature1.js]
"use strict";
var a = {
    /** own x*/
    x: 0
};
