//// [tests/cases/conformance/es6/computedProperties/computedPropertyNamesDeclarationEmit6_ES6.ts] ////

//// [computedPropertyNamesDeclarationEmit6_ES6.ts]
var v = {
  [-1]: {},
  [+1]: {},
  [~1]: {},
  [!1]: {}
}


//// [computedPropertyNamesDeclarationEmit6_ES6.js]
"use strict";
var v = {
    [-1]: {},
    [+1]: {},
    [~1]: {},
    [!1]: {}
};


//// [computedPropertyNamesDeclarationEmit6_ES6.d.ts]
declare var v: {
    [x: number]: {};
    [-1]: {};
    1: {};
};
