//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of13.ts] ////

//// [ES5For-of13.ts]
for (let v of ['a', 'b', 'c']) {
    var x = v;
}

//// [ES5For-of13.js]
"use strict";
for (let v of ['a', 'b', 'c']) {
    var x = v;
}
//# sourceMappingURL=ES5For-of13.js.map