//// [tests/cases/conformance/async/es6/asyncArrowFunction/asyncArrowFunction1_es6.ts] ////

//// [asyncArrowFunction1_es6.ts]
var foo = async (): Promise<void> => {
};

//// [asyncArrowFunction1_es6.js]
"use strict";
var foo = () => __awaiter(void 0, void 0, void 0, function* () {
});
