//// [tests/cases/compiler/commentOnArrayElement6.ts] ////

//// [commentOnArrayElement6.ts]
const array = [1, /* comment */];


//// [commentOnArrayElement6.js]
"use strict";
const array = [1, /* comment */];
