//// [tests/cases/compiler/es5andes6module.ts] ////

//// [es5andes6module.ts]
export default class A
{
    constructor ()
    {

    }

    public B()
    {
        return 42;
    }
}


//// [es5andes6module.js]
export default class A {
    constructor() {
    }
    B() {
        return 42;
    }
}
