//// [tests/cases/conformance/classes/classDeclarations/classAndInterfaceMergeConflictingMembers.ts] ////

//// [classAndInterfaceMergeConflictingMembers.ts]
declare class C1 {
    public x : number;
}

interface C1 {
    x : number;
}

declare class C2 {
    protected x : number;
}

interface C2 {
    x : number;
}

declare class C3 {
    private x : number;
}

interface C3 {
    x : number;
}

//// [classAndInterfaceMergeConflictingMembers.js]
"use strict";
