//// [tests/cases/conformance/types/typeRelationships/assignmentCompatibility/assignmentCompatWithWithGenericConstructSignatures.ts] ////

//// [assignmentCompatWithWithGenericConstructSignatures.ts]


//// [assignmentCompatWithWithGenericConstructSignatures.js]
"use strict";
