//// [tests/cases/compiler/es5-commonjs5.ts] ////

//// [es5-commonjs5.ts]
export default function () {
	return "test";
}


//// [es5-commonjs5.js]
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
function default_1() {
    return "test";
}
