//// [tests/cases/conformance/async/es5/functionDeclarations/asyncFunctionDeclaration2_es5.ts] ////

//// [asyncFunctionDeclaration2_es5.ts]
function f(await) {
}

//// [asyncFunctionDeclaration2_es5.js]
"use strict";
function f(await) {
}
