//// [tests/cases/conformance/classes/classDeclarations/classWithPredefinedTypesAsNames2.ts] ////

//// [classWithPredefinedTypesAsNames2.ts]
// classes cannot use predefined types as names

class void {}

//// [classWithPredefinedTypesAsNames2.js]
"use strict";
// classes cannot use predefined types as names
class {
}
void {};
