Fs::
//// [/bower_components/b.ts]


//// [/d.ts]


//// [/folder/e.ts]


//// [/jspm_packages/c.ts]


//// [/node_modules/a.ts]


//// [/tsconfig.json]
{}


configFileName:: tsconfig.json
FileNames::
/d.ts,/folder/e.ts
Errors::

