//// [tests/cases/compiler/contextualTypeObjectSpreadExpression.ts] ////

//// [contextualTypeObjectSpreadExpression.ts]
interface I {
    a: "a";
}
let i: I;
i = { ...{ a: "a" } };


//// [contextualTypeObjectSpreadExpression.js]
"use strict";
let i;
i = Object.assign({ a: "a" });
