//// [tests/cases/compiler/es5-commonjs3.ts] ////

//// [es5-commonjs3.ts]
export default "test";
export var __esModule = 1;


//// [es5-commonjs3.js]
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.__esModule = void 0;
exports.default = "test";
exports.__esModule = 1;
