//// [tests/cases/compiler/aliasInaccessibleModule2.ts] ////

//// [aliasInaccessibleModule2.ts]
namespace M {
    namespace N {
        class C {
        }
        
    }
    import R = N;
    export import X = R;
}

//// [aliasInaccessibleModule2.js]
"use strict";
var M;
(function (M) {
    let N;
    (function (N) {
        class C {
        }
    })(N || (N = {}));
    var R = N;
    M.X = R;
})(M || (M = {}));


//// [aliasInaccessibleModule2.d.ts]
declare namespace M {
    namespace N {
    }
    import R = N;
    export import X = R;
    export {};
}
