//// [tests/cases/compiler/continueTarget3.ts] ////

//// [continueTarget3.ts]
target1:
target2:
while (true) {
  continue target1;
}

//// [continueTarget3.js]
"use strict";
target1: target2: while (true) {
    continue target1;
}
