// [source.js.map]
{"version":3,"file":"source.js","sourceRoot":"","sources":["source.ts","another.html"],"names":[],"mappings":";AAAA;yBACyB,CAAA;ACDzB,QAAS,CDED"}

// [source.js]
"use strict";
`multi
                    line`;
'change';
//# sourceMappingURL=source.js.map