//// [tests/cases/compiler/controlFlowManyConsecutiveConditionsNoTimeout.ts] ////

//// [controlFlowManyConsecutiveConditionsNoTimeout.ts]
export enum Choice {
    One,
    Two,
}

const choice: Choice = Choice.One;
const choiceOne = Choice.One;

if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {}
if (choice === choiceOne) {} 

while (true) {
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {}
    if (choice === choiceOne) {} 
}


//// [controlFlowManyConsecutiveConditionsNoTimeout.js]
export var Choice;
(function (Choice) {
    Choice[Choice["One"] = 0] = "One";
    Choice[Choice["Two"] = 1] = "Two";
})(Choice || (Choice = {}));
const choice = Choice.One;
const choiceOne = Choice.One;
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
if (choice === choiceOne) { }
while (true) {
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
    if (choice === choiceOne) { }
}
