//// [tests/cases/conformance/es6/classExpressions/classExpressionES62.ts] ////

//// [classExpressionES62.ts]
class D { }
var v = class C extends D {};

//// [classExpressionES62.js]
"use strict";
class D {
}
var v = class C extends D {
};
