//// [tests/cases/compiler/crashInsourcePropertyIsRelatableToTargetProperty.ts] ////

//// [crashInsourcePropertyIsRelatableToTargetProperty.ts]
class C {
    private x = 1;
}
class D extends C { }
function foo(x: "hi", items: string[]): typeof foo;
function foo(x: string, items: string[]): typeof foo {
    return null;
}
var a: D = foo("hi", []);


//// [crashInsourcePropertyIsRelatableToTargetProperty.js]
"use strict";
class C {
    constructor() {
        this.x = 1;
    }
}
class D extends C {
}
function foo(x, items) {
    return null;
}
var a = foo("hi", []);
