0.ts --fallbackPolling
CompilerOptions::
{}
WatchOptions::
{}
FileNames::
0.ts
Errors::
error TS5080: Watch option 'fallbackPolling' requires a value of type string.
error TS6046: Argument for '--fallbackPolling' option must be: 'fixedinterval', 'priorityinterval', 'dynamicpriority', 'fixedchunksize'.
