0.ts --target
CompilerOptions::
{}
WatchOptions::

FileNames::
0.ts
Errors::
error TS6044: Compiler option 'target' expects an argument.
error TS6046: Argument for '--target' option must be: 'es6', 'es2015', 'es2016', 'es2017', 'es2018', 'es2019', 'es2020', 'es2021', 'es2022', 'es2023', 'es2024', 'es2025', 'esnext'.
