//// [tests/cases/compiler/deleteOperator1.ts] ////

//// [deleteOperator1.ts]
var a;
var x: boolean = delete a;
var y: any = delete a;
var z: number = delete a;

//// [deleteOperator1.js]
"use strict";
var a;
var x = delete a;
var y = delete a;
var z = delete a;
