//// [tests/cases/compiler/declarationEmitDefaultExport2.ts] ////

//// [declarationEmitDefaultExport2.ts]
export default class {
}

//// [declarationEmitDefaultExport2.js]
export default class {
}


//// [declarationEmitDefaultExport2.d.ts]
export default class {
}
