//// [tests/cases/conformance/es6/yieldExpressions/YieldStarExpression4_es6.ts] ////

//// [YieldStarExpression4_es6.ts]
function *g() {
    yield * [];
}

//// [YieldStarExpression4_es6.js]
"use strict";
function* g() {
    yield* [];
}
