//// [tests/cases/conformance/es6/destructuring/destructuringParameterDeclaration7ES5iterable.ts] ////

//// [destructuringParameterDeclaration7ES5iterable.ts]
interface ISomething {
    foo: string,
    bar: string
}

function foo({}, {foo, bar}: ISomething) {}

function baz([], {foo, bar}: ISomething) {}

function one([], {}) {}

function two([], [a, b, c]: number[]) {}


//// [destructuringParameterDeclaration7ES5iterable.js]
"use strict";
function foo({}, { foo, bar }) { }
function baz([], { foo, bar }) { }
function one([], {}) { }
function two([], [a, b, c]) { }
