//// [tests/cases/compiler/cannotInvokeNewOnIndexExpression.ts] ////

//// [cannotInvokeNewOnIndexExpression.ts]
var test: any[] = new any[1];

//// [cannotInvokeNewOnIndexExpression.js]
"use strict";
var test = new any[1];
