//// [tests/cases/conformance/classes/propertyMemberDeclarations/accessorsOverrideProperty3.ts] ////

//// [accessorsOverrideProperty3.ts]
declare class Animal {
    sound: string
}
class Lion extends Animal {
    _sound = 'grrr'
    get sound() { return this._sound } // error here
    set sound(val) { this._sound = val }
}


//// [accessorsOverrideProperty3.js]
"use strict";
class Lion extends Animal {
    _sound = 'grrr';
    get sound() { return this._sound; } // error here
    set sound(val) { this._sound = val; }
}
