//// [tests/cases/conformance/async/es2017/asyncArrowFunction/asyncArrowFunction2_es2017.ts] ////

//// [asyncArrowFunction2_es2017.ts]
var f = (await) => {
}

//// [asyncArrowFunction2_es2017.js]
"use strict";
var f = (await) => {
};
