//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of26.ts] ////

//// [ES5For-of26.ts]
for (var [a = 0, b = 1] of [2, 3]) {
    a;
    b;
}

//// [ES5For-of26.js]
"use strict";
for (var [a = 0, b = 1] of [2, 3]) {
    a;
    b;
}
//# sourceMappingURL=ES5For-of26.js.map