--build --clean --dry --force --verbose
CompilerOptions::
{}
WatchOptions::

FileNames::

Errors::
error TS6369: Option '--build' must be the first command line argument.
error TS5093: Compiler option '--clean' may only be used with '--build'.
error TS5093: Compiler option '--dry' may only be used with '--build'.
error TS5093: Compiler option '--force' may only be used with '--build'.
error TS5093: Compiler option '--verbose' may only be used with '--build'.
