//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of18.ts] ////

//// [ES5For-of18.ts]
for (let v of []) {
    v;
}
for (let v of []) {
    v;
}


//// [ES5For-of18.js]
"use strict";
for (let v of []) {
    v;
}
for (let v of []) {
    v;
}
