//// [tests/cases/conformance/ambient/ambientShorthand_declarationEmit.ts] ////

//// [ambientShorthand_declarationEmit.ts]
declare module "foo";


//// [ambientShorthand_declarationEmit.js]
"use strict";


//// [ambientShorthand_declarationEmit.d.ts]
declare module "foo";
