//// [tests/cases/conformance/es6/computedProperties/computedPropertyNames20_ES6.ts] ////

//// [computedPropertyNames20_ES6.ts]
var obj = {
    [this.bar]: 0
}

//// [computedPropertyNames20_ES6.js]
"use strict";
var obj = {
    [this.bar]: 0
};
