0.ts --newLine
CompilerOptions::
{}
WatchOptions::

FileNames::
0.ts
Errors::
error TS6044: Compiler option 'newLine' expects an argument.
error TS6046: Argument for '--newLine' option must be: 'crlf', 'lf'.
