--excludeDirectories **/../*
buildOptions::
{}
WatchOptions::
{
  "excludeDirectories": []
}
Projects::
.
Errors::
error TS5065: File specification cannot contain a parent directory ('..') that appears after a recursive directory wildcard ('**'): '**/../*'.
