//// [tests/cases/conformance/es6/destructuring/emptyVariableDeclarationBindingPatterns02_ES6.ts] ////

//// [emptyVariableDeclarationBindingPatterns02_ES6.ts]
(function () {
    var {};
    let {};
    const {};

    var [];
    let [];
    const [];
})();

//// [emptyVariableDeclarationBindingPatterns02_ES6.js]
"use strict";
(function () {
    var {};
    let {};
    const {};
    var [];
    let [];
    const [];
})();


//// [emptyVariableDeclarationBindingPatterns02_ES6.d.ts]
