//// [tests/cases/compiler/duplicatePropertiesInStrictMode.ts] ////

//// [duplicatePropertiesInStrictMode.ts]
"use strict";
var x = {
  x: 1,
  x: 2
}

//// [duplicatePropertiesInStrictMode.js]
"use strict";
var x = {
    x: 1,
    x: 2
};
