--excludeFiles **/../* 0.ts
CompilerOptions::
{}
WatchOptions::
{
  "excludeFiles": []
}
FileNames::
0.ts
Errors::
error TS5065: File specification cannot contain a parent directory ('..') that appears after a recursive directory wildcard ('**'): '**/../*'.
