//// [tests/cases/compiler/es5-asyncFunctionObjectLiterals.ts] ////

//// [es5-asyncFunctionObjectLiterals.ts]
declare var x, y, z, a, b, c;

async function objectLiteral0() {
    x = {
        a: await y,
        b: z
    };
}

async function objectLiteral1() {
    x = {
        a: y,
        b: await z
    };
}

async function objectLiteral2() {
    x = {
        [await a]: y,
        b: z
    };
}

async function objectLiteral3() {
    x = {
        [a]: await y,
        b: z
    };
}

async function objectLiteral4() {
    x = {
        a: await y,
        [b]: z
    };
}

async function objectLiteral5() {
    x = {
        a: y,
        [await b]: z
    };
}

async function objectLiteral6() {
    x = {
        a: y,
        [b]: await z
    };
}

//// [es5-asyncFunctionObjectLiterals.js]
"use strict";
function objectLiteral0() {
    return __awaiter(this, void 0, void 0, function* () {
        x = {
            a: yield y,
            b: z
        };
    });
}
function objectLiteral1() {
    return __awaiter(this, void 0, void 0, function* () {
        x = {
            a: y,
            b: yield z
        };
    });
}
function objectLiteral2() {
    return __awaiter(this, void 0, void 0, function* () {
        x = {
            [yield a]: y,
            b: z
        };
    });
}
function objectLiteral3() {
    return __awaiter(this, void 0, void 0, function* () {
        x = {
            [a]: yield y,
            b: z
        };
    });
}
function objectLiteral4() {
    return __awaiter(this, void 0, void 0, function* () {
        x = {
            a: yield y,
            [b]: z
        };
    });
}
function objectLiteral5() {
    return __awaiter(this, void 0, void 0, function* () {
        x = {
            a: y,
            [yield b]: z
        };
    });
}
function objectLiteral6() {
    return __awaiter(this, void 0, void 0, function* () {
        x = {
            a: y,
            [b]: yield z
        };
    });
}
