//// [tests/cases/compiler/commentOnArrayElement10.ts] ////

//// [commentOnArrayElement10.ts]
const array = [,, /* comment */];


//// [commentOnArrayElement10.js]
"use strict";
const array = [, , /* comment */];
