0.ts --jsx
CompilerOptions::
{}
WatchOptions::

FileNames::
0.ts
Errors::
error TS6044: Compiler option 'jsx' expects an argument.
error TS6046: Argument for '--jsx' option must be: 'preserve', 'react-native', 'react-jsx', 'react-jsxdev', 'react'.
