Input::
{
            "exclude": [
                "xx//file.d.ts"
            ]
        }
Config::
{
  "exclude": [
    "xx//file.d.ts"
  ]
}
Errors::


Input::
{
            "exclude": [
                "xx/*file.d.ts*/"
            ]
        }
Config::
{
  "exclude": [
    "xx/*file.d.ts*/"
  ]
}
Errors::

