//// [tests/cases/conformance/es6/arrowFunction/emitArrowFunctionWhenUsingArguments12.ts] ////

//// [emitArrowFunctionWhenUsingArguments12.ts]
class C {
    f(arguments) {
        var a = () => arguments;
    }
}

//// [emitArrowFunctionWhenUsingArguments12.js]
"use strict";
var C = /** @class */ (function () {
    function C() {
    }
    C.prototype.f = function (arguments) {
        var a = function () { return arguments; };
    };
    return C;
}());
