//// [tests/cases/compiler/conditionalExpressionNewLine3.ts] ////

//// [conditionalExpressionNewLine3.ts]
var v = a ?
  b : c;

//// [conditionalExpressionNewLine3.js]
"use strict";
var v = a ?
    b : c;
