Fs::
//// [/apath/..c.ts]


//// [/apath/.b.ts]


//// [/apath/.git/a.ts]


//// [/apath/test.ts]


//// [/apath/tsconfig.json]
{}


configFileName:: tsconfig.json
FileNames::
/apath/test.ts
Errors::

