//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of25.ts] ////

//// [ES5For-of25.ts]
var a = [1, 2, 3];
for (var v of a) {
    v;
    a;
}

//// [ES5For-of25.js]
"use strict";
var a = [1, 2, 3];
for (var v of a) {
    v;
    a;
}
//# sourceMappingURL=ES5For-of25.js.map