//// [tests/cases/compiler/collisionExportsRequireAndInternalModuleAliasInGlobalFile.ts] ////

//// [collisionExportsRequireAndInternalModuleAliasInGlobalFile.ts]
namespace mOfGloalFile {
    export class c {
    }
}
import exports = mOfGloalFile.c;
import require = mOfGloalFile.c;
new exports();
new require();

namespace m1 {
    import exports = mOfGloalFile.c;
    import require = mOfGloalFile.c;
    new exports();
    new require();
}

namespace m2 {
    export import exports = mOfGloalFile.c;
    export import require = mOfGloalFile.c;
    new exports();
    new require();
}

//// [collisionExportsRequireAndInternalModuleAliasInGlobalFile.js]
"use strict";
var mOfGloalFile;
(function (mOfGloalFile) {
    class c {
    }
    mOfGloalFile.c = c;
})(mOfGloalFile || (mOfGloalFile = {}));
var exports = mOfGloalFile.c;
var require = mOfGloalFile.c;
new exports();
new require();
var m1;
(function (m1) {
    var exports = mOfGloalFile.c;
    var require = mOfGloalFile.c;
    new exports();
    new require();
})(m1 || (m1 = {}));
var m2;
(function (m2) {
    m2.exports = mOfGloalFile.c;
    m2.require = mOfGloalFile.c;
    new m2.exports();
    new m2.require();
})(m2 || (m2 = {}));
