//// [tests/cases/compiler/constDeclarations-es5.ts] ////

//// [constDeclarations-es5.ts]
const z7 = false;
const z8: number = 23;
const z9 = 0, z10 :string = "", z11 = null;


//// [constDeclarations-es5.js]
"use strict";
var z7 = false;
var z8 = 23;
var z9 = 0, z10 = "", z11 = null;
