//// [tests/cases/conformance/statements/for-ofStatements/ES5For-ofTypeCheck3.ts] ////

//// [ES5For-ofTypeCheck3.ts]
var tuple: [string, number] = ["", 0];
for (var v of tuple) { }

//// [ES5For-ofTypeCheck3.js]
"use strict";
var tuple = ["", 0];
for (var v of tuple) { }
