//// [tests/cases/conformance/async/es6/asyncConstructor_es6.ts] ////

//// [asyncConstructor_es6.ts]
class C {  
  async constructor() {    
  }
}

//// [asyncConstructor_es6.js]
"use strict";
class C {
    constructor() {
    }
}
