//// [tests/cases/compiler/declarationEmitDefaultExport1.ts] ////

//// [declarationEmitDefaultExport1.ts]
export default class C {
}

//// [declarationEmitDefaultExport1.js]
export default class C {
}


//// [declarationEmitDefaultExport1.d.ts]
export default class C {
}
