//// [tests/cases/compiler/arithmeticOnInvalidTypes.ts] ////

//// [arithmeticOnInvalidTypes.ts]
var x: Number;
var y: Number;
var z = x + y;
var z2 = x - y;
var z3 = x * y;
var z4 = x / y;

//// [arithmeticOnInvalidTypes.js]
"use strict";
var x;
var y;
var z = x + y;
var z2 = x - y;
var z3 = x * y;
var z4 = x / y;
