//// [tests/cases/compiler/contextualTyping29.ts] ////

//// [contextualTyping29.ts]
function foo(param:number[]){}; foo([1, 3]);

//// [contextualTyping29.js]
"use strict";
function foo(param) { }
;
foo([1, 3]);
