0.ts --strictNullChecks --paths
CompilerOptions::
{
  "strictNullChecks": true
}
WatchOptions::

FileNames::
0.ts
Errors::
error TS6064: Option 'paths' can only be specified in 'tsconfig.json' file or set to 'null' on command line.
