//// [tests/cases/conformance/es6/classDeclaration/classWithSemicolonClassElementES62.ts] ////

//// [classWithSemicolonClassElementES62.ts]
class C {
    ;
    ;
}

//// [classWithSemicolonClassElementES62.js]
"use strict";
class C {
    ;
    ;
}
