//// [tests/cases/compiler/duplicateIdentifierDifferentSpelling.ts] ////

//// [duplicateIdentifierDifferentSpelling.ts]
class A {
  0b11 = '';
  3 = '';
}

var X = { 0b11: '', 3: '' };


//// [duplicateIdentifierDifferentSpelling.js]
"use strict";
class A {
    constructor() {
        this[0b11] = '';
        this[3] = '';
    }
}
var X = { 0b11: '', 3: '' };
