//// [tests/cases/compiler/conditionalExpressionNewLine1.ts] ////

//// [conditionalExpressionNewLine1.ts]
var v = a ? b : c;

//// [conditionalExpressionNewLine1.js]
"use strict";
var v = a ? b : c;
