Fs::
//// [/apath/a.ts]


//// [/apath/tsconfig.json]
{
                "files": [],
                "references": []
            }


configFileName:: /apath/tsconfig.json
FileNames::

Errors::
[91merror[0m[90m TS18002: [0mThe 'files' list in config file '/apath/tsconfig.json' is empty.

