//// [tests/cases/compiler/complexNarrowingWithAny.ts] ////

//// [complexNarrowingWithAny.ts]
// Repro from #10869

/**
 * This file is generated by the Angular 2 template compiler.
 * Do not edit.
 */
 /* tslint:disable */

// import * as import0 from '@angular/core/src/linker/ng_module_factory';
// import * as import1 from '../../app';
// import * as import2 from '@angular/common/src/common_module';
// import * as import3 from '@angular/core/src/application_module';
// import * as import4 from '@angular/platform-browser/src/browser';
// import * as import5 from '@angular/forms/src/directives';
// import * as import6 from '@angular/forms/src/form_providers';
// import * as import7 from '@angular/common/src/localization';
// import * as import8 from '@angular/core/src/application_init';
// import * as import9 from '@angular/core/src/testability/testability';
// import * as import10 from '@angular/core/src/application_ref';
// import * as import11 from '@angular/core/src/linker/compiler';
// import * as import12 from '@angular/platform-browser/src/dom/events/hammer_gestures';
// import * as import13 from '@angular/platform-browser/src/dom/events/event_manager';
// import * as import14 from '@angular/platform-browser/src/dom/shared_styles_host';
// import * as import15 from '@angular/platform-browser/src/dom/dom_renderer';
// import * as import16 from '@angular/platform-browser/src/security/dom_sanitization_service';
// import * as import17 from '@angular/core/src/linker/view_utils';
// import * as import18 from '@angular/forms/src/form_builder';
// import * as import19 from '@angular/forms/src/directives/radio_control_value_accessor';
// import * as import20 from '@angular/core/src/di/injector';
// import * as import21 from '@angular/core/src/application_tokens';
// import * as import22 from '@angular/platform-browser/src/dom/events/dom_events';
// import * as import23 from '@angular/platform-browser/src/dom/events/key_events';
// import * as import24 from '@angular/core/src/zone/ng_zone';
// import * as import25 from '@angular/platform-browser/src/dom/debug/ng_probe';
// import * as import26 from '@angular/core/src/console';
// import * as import27 from '@angular/core/src/i18n/tokens';
// import * as import28 from '@angular/core/src/error_handler';
// import * as import29 from '@angular/platform-browser/src/dom/dom_tokens';
// import * as import30 from '@angular/platform-browser/src/dom/animation_driver';
// import * as import31 from '@angular/core/src/render/api';
// import * as import32 from '@angular/core/src/security';
// import * as import33 from '@angular/core/src/change_detection/differs/iterable_differs';
// import * as import34 from '@angular/core/src/change_detection/differs/keyvalue_differs';
// import * as import35 from '@angular/core/src/i18n/tokens';
// import * as import36 from '@angular/core/src/render/api';
// import * as import37 from '@angular/core/src/linker/view';
// import * as import38 from '@angular/core/src/linker/element';
// import * as import39 from '@angular/core/src/linker/view_utils';
// import * as import40 from '@angular/core/src/linker/view_type';
// import * as import41 from '@angular/core/src/change_detection/change_detection';
// import * as import42 from '@angular/core/src/metadata/view';
// import * as import43 from '@angular/core/src/linker/component_factory';
// import * as import44 from '@angular/forms/src/directives/reactive_directives/form_group_directive';
// import * as import45 from '@angular/forms/src/directives/ng_control_status';
// import * as import46 from '@angular/forms/src/directives/default_value_accessor';
// import * as import47 from '@angular/forms/src/directives/reactive_directives/form_control_name';
// import * as import48 from '@angular/core/src/linker/element_ref';
// import * as import49 from '@angular/forms/src/directives/control_value_accessor';
// import * as import50 from '@angular/forms/src/directives/ng_control';
// import * as import51 from '@angular/forms/src/directives/control_container';

//stubbed out imports

namespace import44 {
	export class FormGroupDirective {
		constructor(any){}
	}
}

namespace import45 {
	export class NgControlStatus {
		constructor(any){}
	}
	export class NgControlStatusGroup {
		constructor(any){}
	}
}

namespace import46 {
	export class DefaultValueAccessor {
		constructor(any){}
	}
}

namespace import47 {
	export class FormControlName {
		constructor(any){}
	}
}

namespace import48 {
	export class FormControlName {
		constructor(any){}
	}
}


//HERE BE DRAGONS

//Using a value here - 65+ seconds to typecheck
namespace import49 {
	//real code uses an opaque token, using new String() to simulate.
	//export var NG_VALUE_ACCESSOR = new OpaqueToken('ngValueAccessor')
	export var NG_VALUE_ACCESSOR = new String('foo')
}

//using a class - < 1 sec typecheck
// namespace import49 {
// 	export class NG_VALUE_ACCESSOR {
// 		constructor(any){}
// 	}
// }

//END DRAGONS


namespace import50 {
	export class NgControl {
		constructor(any){}
	}
}

namespace import51 {
	export class ControlContainer {
		constructor(any){}
	}
}



class _View_AppComponent0 {
  _text_0:any;
  _el_1:any;
  _FormGroupDirective_1_3:import44.FormGroupDirective;
  _ControlContainer_1_4:any;
  _NgControlStatusGroup_1_5:import45.NgControlStatusGroup;
  _text_2:any;
  _el_3:any;
  _DefaultValueAccessor_3_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_3_4:any[];
  _FormControlName_3_5:import47.FormControlName;
  _NgControl_3_6:any;
  _NgControlStatus_3_7:import45.NgControlStatus;
  _el_4:any;
  _text_5:any;
  _el_6:any;
  _DefaultValueAccessor_6_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_6_4:any[];
  _FormControlName_6_5:import47.FormControlName;
  _NgControl_6_6:any;
  _NgControlStatus_6_7:import45.NgControlStatus;
  _el_7:any;
  _text_8:any;
  _el_9:any;
  _DefaultValueAccessor_9_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_9_4:any[];
  _FormControlName_9_5:import47.FormControlName;
  _NgControl_9_6:any;
  _NgControlStatus_9_7:import45.NgControlStatus;
  _el_10:any;
  _text_11:any;
  _el_12:any;
  _DefaultValueAccessor_12_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_12_4:any[];
  _FormControlName_12_5:import47.FormControlName;
  _NgControl_12_6:any;
  _NgControlStatus_12_7:import45.NgControlStatus;
  _el_13:any;
  _text_14:any;
  _el_15:any;
  _DefaultValueAccessor_15_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_15_4:any[];
  _FormControlName_15_5:import47.FormControlName;
  _NgControl_15_6:any;
  _NgControlStatus_15_7:import45.NgControlStatus;
  _el_16:any;
  _text_17:any;
  _el_18:any;
  _DefaultValueAccessor_18_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_18_4:any[];
  _FormControlName_18_5:import47.FormControlName;
  _NgControl_18_6:any;
  _NgControlStatus_18_7:import45.NgControlStatus;
  _el_19:any;
  _text_20:any;
  _el_21:any;
  _DefaultValueAccessor_21_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_21_4:any[];
  _FormControlName_21_5:import47.FormControlName;
  _NgControl_21_6:any;
  _NgControlStatus_21_7:import45.NgControlStatus;
  _el_22:any;
  _text_23:any;
  _el_24:any;
  _DefaultValueAccessor_24_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_24_4:any[];
  _FormControlName_24_5:import47.FormControlName;
  _NgControl_24_6:any;
  _NgControlStatus_24_7:import45.NgControlStatus;
  _el_25:any;
  _text_26:any;
  _el_27:any;
  _DefaultValueAccessor_27_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_27_4:any[];
  _FormControlName_27_5:import47.FormControlName;
  _NgControl_27_6:any;
  _NgControlStatus_27_7:import45.NgControlStatus;
  _el_28:any;
  _text_29:any;
  _el_30:any;
  _DefaultValueAccessor_30_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_30_4:any[];
  _FormControlName_30_5:import47.FormControlName;
  _NgControl_30_6:any;
  _NgControlStatus_30_7:import45.NgControlStatus;
  _el_31:any;
  _text_32:any;
  _el_33:any;
  _DefaultValueAccessor_33_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_33_4:any[];
  _FormControlName_33_5:import47.FormControlName;
  _NgControl_33_6:any;
  _NgControlStatus_33_7:import45.NgControlStatus;
  _el_34:any;
  _text_35:any;
  _el_36:any;
  _DefaultValueAccessor_36_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_36_4:any[];
  _FormControlName_36_5:import47.FormControlName;
  _NgControl_36_6:any;
  _NgControlStatus_36_7:import45.NgControlStatus;
  _el_37:any;
  _text_38:any;
  _el_39:any;
  _DefaultValueAccessor_39_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_39_4:any[];
  _FormControlName_39_5:import47.FormControlName;
  _NgControl_39_6:any;
  _NgControlStatus_39_7:import45.NgControlStatus;
  _el_40:any;
  _text_41:any;
  _el_42:any;
  _DefaultValueAccessor_42_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_42_4:any[];
  _FormControlName_42_5:import47.FormControlName;
  _NgControl_42_6:any;
  _NgControlStatus_42_7:import45.NgControlStatus;
  _el_43:any;
  _text_44:any;
  _el_45:any;
  _DefaultValueAccessor_45_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_45_4:any[];
  _FormControlName_45_5:import47.FormControlName;
  _NgControl_45_6:any;
  _NgControlStatus_45_7:import45.NgControlStatus;
  _el_46:any;
  _text_47:any;
  _el_48:any;
  _DefaultValueAccessor_48_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_48_4:any[];
  _FormControlName_48_5:import47.FormControlName;
  _NgControl_48_6:any;
  _NgControlStatus_48_7:import45.NgControlStatus;
  _el_49:any;
  _text_50:any;
  _el_51:any;
  _DefaultValueAccessor_51_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_51_4:any[];
  _FormControlName_51_5:import47.FormControlName;
  _NgControl_51_6:any;
  _NgControlStatus_51_7:import45.NgControlStatus;
  _el_52:any;
  _text_53:any;
  _el_54:any;
  _DefaultValueAccessor_54_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_54_4:any[];
  _FormControlName_54_5:import47.FormControlName;
  _NgControl_54_6:any;
  _NgControlStatus_54_7:import45.NgControlStatus;
  _el_55:any;
  _text_56:any;
  _el_57:any;
  _DefaultValueAccessor_57_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_57_4:any[];
  _FormControlName_57_5:import47.FormControlName;
  _NgControl_57_6:any;
  _NgControlStatus_57_7:import45.NgControlStatus;
  _el_58:any;
  _text_59:any;
  _el_60:any;
  _DefaultValueAccessor_60_3:import46.DefaultValueAccessor;
  _NG_VALUE_ACCESSOR_60_4:any[];
  _FormControlName_60_5:import47.FormControlName;
  _NgControl_60_6:any;
  _NgControlStatus_60_7:import45.NgControlStatus;
  _el_61:any;
  _text_62:any;
  _text_63:any;
  /*private*/ _expr_2:any;
  /*private*/ _expr_3:any;
  /*private*/ _expr_4:any;
  /*private*/ _expr_5:any;
  /*private*/ _expr_6:any;
  /*private*/ _expr_7:any;
  /*private*/ _expr_8:any;
  /*private*/ _expr_11:any;
  /*private*/ _expr_12:any;
  /*private*/ _expr_13:any;
  /*private*/ _expr_14:any;
  /*private*/ _expr_15:any;
  /*private*/ _expr_16:any;
  /*private*/ _expr_17:any;
  /*private*/ _expr_20:any;
  /*private*/ _expr_21:any;
  /*private*/ _expr_22:any;
  /*private*/ _expr_23:any;
  /*private*/ _expr_24:any;
  /*private*/ _expr_25:any;
  /*private*/ _expr_26:any;
  /*private*/ _expr_29:any;
  /*private*/ _expr_30:any;
  /*private*/ _expr_31:any;
  /*private*/ _expr_32:any;
  /*private*/ _expr_33:any;
  /*private*/ _expr_34:any;
  /*private*/ _expr_35:any;
  /*private*/ _expr_38:any;
  /*private*/ _expr_39:any;
  /*private*/ _expr_40:any;
  /*private*/ _expr_41:any;
  /*private*/ _expr_42:any;
  /*private*/ _expr_43:any;
  /*private*/ _expr_44:any;
  /*private*/ _expr_47:any;
  /*private*/ _expr_48:any;
  /*private*/ _expr_49:any;
  /*private*/ _expr_50:any;
  /*private*/ _expr_51:any;
  /*private*/ _expr_52:any;
  /*private*/ _expr_53:any;
  /*private*/ _expr_56:any;
  /*private*/ _expr_57:any;
  /*private*/ _expr_58:any;
  /*private*/ _expr_59:any;
  /*private*/ _expr_60:any;
  /*private*/ _expr_61:any;
  /*private*/ _expr_62:any;
  /*private*/ _expr_65:any;
  /*private*/ _expr_66:any;
  /*private*/ _expr_67:any;
  /*private*/ _expr_68:any;
  /*private*/ _expr_69:any;
  /*private*/ _expr_70:any;
  /*private*/ _expr_71:any;
  /*private*/ _expr_74:any;
  /*private*/ _expr_75:any;
  /*private*/ _expr_76:any;
  /*private*/ _expr_77:any;
  /*private*/ _expr_78:any;
  /*private*/ _expr_79:any;
  /*private*/ _expr_80:any;
  /*private*/ _expr_83:any;
  /*private*/ _expr_84:any;
  /*private*/ _expr_85:any;
  /*private*/ _expr_86:any;
  /*private*/ _expr_87:any;
  /*private*/ _expr_88:any;
  /*private*/ _expr_89:any;
  /*private*/ _expr_92:any;
  /*private*/ _expr_93:any;
  /*private*/ _expr_94:any;
  /*private*/ _expr_95:any;
  /*private*/ _expr_96:any;
  /*private*/ _expr_97:any;
  /*private*/ _expr_98:any;
  /*private*/ _expr_101:any;
  /*private*/ _expr_102:any;
  /*private*/ _expr_103:any;
  /*private*/ _expr_104:any;
  /*private*/ _expr_105:any;
  /*private*/ _expr_106:any;
  /*private*/ _expr_107:any;
  /*private*/ _expr_110:any;
  /*private*/ _expr_111:any;
  /*private*/ _expr_112:any;
  /*private*/ _expr_113:any;
  /*private*/ _expr_114:any;
  /*private*/ _expr_115:any;
  /*private*/ _expr_116:any;
  /*private*/ _expr_119:any;
  /*private*/ _expr_120:any;
  /*private*/ _expr_121:any;
  /*private*/ _expr_122:any;
  /*private*/ _expr_123:any;
  /*private*/ _expr_124:any;
  /*private*/ _expr_125:any;
  /*private*/ _expr_128:any;
  /*private*/ _expr_129:any;
  /*private*/ _expr_130:any;
  /*private*/ _expr_131:any;
  /*private*/ _expr_132:any;
  /*private*/ _expr_133:any;
  /*private*/ _expr_134:any;
  /*private*/ _expr_137:any;
  /*private*/ _expr_138:any;
  /*private*/ _expr_139:any;
  /*private*/ _expr_140:any;
  /*private*/ _expr_141:any;
  /*private*/ _expr_142:any;
  /*private*/ _expr_143:any;
  /*private*/ _expr_146:any;
  /*private*/ _expr_147:any;
  /*private*/ _expr_148:any;
  /*private*/ _expr_149:any;
  /*private*/ _expr_150:any;
  /*private*/ _expr_151:any;
  /*private*/ _expr_152:any;
  /*private*/ _expr_155:any;
  /*private*/ _expr_156:any;
  /*private*/ _expr_157:any;
  /*private*/ _expr_158:any;
  /*private*/ _expr_159:any;
  /*private*/ _expr_160:any;
  /*private*/ _expr_161:any;
  /*private*/ _expr_164:any;
  /*private*/ _expr_165:any;
  /*private*/ _expr_166:any;
  /*private*/ _expr_167:any;
  /*private*/ _expr_168:any;
  /*private*/ _expr_169:any;
  /*private*/ _expr_170:any;
  /*private*/ _expr_173:any;
  /*private*/ _expr_174:any;
  /*private*/ _expr_175:any;
  /*private*/ _expr_176:any;
  /*private*/ _expr_177:any;
  /*private*/ _expr_178:any;
  /*private*/ _expr_179:any;
  /*private*/ _expr_182:any;
  /*private*/ _expr_183:any;
  /*private*/ _expr_184:any;
  /*private*/ _expr_185:any;
  /*private*/ _expr_186:any;
  /*private*/ _expr_187:any;
  /*private*/ _expr_188:any;
  constructor(viewUtils:any,parentInjector:any,declarationEl:any) {
  }

  injectorGetInternal(token:any,requestNodeIndex:number,notFoundResult:any):any {
    if (((token === import46.DefaultValueAccessor) && (3 === requestNodeIndex))) { return this._DefaultValueAccessor_3_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (3 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_3_4; }
    if (((token === import47.FormControlName) && (3 === requestNodeIndex))) { return this._FormControlName_3_5; }
    if (((token === import50.NgControl) && (3 === requestNodeIndex))) { return this._NgControl_3_6; }
    if (((token === import45.NgControlStatus) && (3 === requestNodeIndex))) { return this._NgControlStatus_3_7; }
    if (((token === import46.DefaultValueAccessor) && (6 === requestNodeIndex))) { return this._DefaultValueAccessor_6_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (6 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_6_4; }
    if (((token === import47.FormControlName) && (6 === requestNodeIndex))) { return this._FormControlName_6_5; }
    if (((token === import50.NgControl) && (6 === requestNodeIndex))) { return this._NgControl_6_6; }
    if (((token === import45.NgControlStatus) && (6 === requestNodeIndex))) { return this._NgControlStatus_6_7; }
    if (((token === import46.DefaultValueAccessor) && (9 === requestNodeIndex))) { return this._DefaultValueAccessor_9_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (9 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_9_4; }
    if (((token === import47.FormControlName) && (9 === requestNodeIndex))) { return this._FormControlName_9_5; }
    if (((token === import50.NgControl) && (9 === requestNodeIndex))) { return this._NgControl_9_6; }
    if (((token === import45.NgControlStatus) && (9 === requestNodeIndex))) { return this._NgControlStatus_9_7; }
    if (((token === import46.DefaultValueAccessor) && (12 === requestNodeIndex))) { return this._DefaultValueAccessor_12_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (12 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_12_4; }
    if (((token === import47.FormControlName) && (12 === requestNodeIndex))) { return this._FormControlName_12_5; }
    if (((token === import50.NgControl) && (12 === requestNodeIndex))) { return this._NgControl_12_6; }
    if (((token === import45.NgControlStatus) && (12 === requestNodeIndex))) { return this._NgControlStatus_12_7; }
    if (((token === import46.DefaultValueAccessor) && (15 === requestNodeIndex))) { return this._DefaultValueAccessor_15_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (15 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_15_4; }
    if (((token === import47.FormControlName) && (15 === requestNodeIndex))) { return this._FormControlName_15_5; }
    if (((token === import50.NgControl) && (15 === requestNodeIndex))) { return this._NgControl_15_6; }
    if (((token === import45.NgControlStatus) && (15 === requestNodeIndex))) { return this._NgControlStatus_15_7; }
    if (((token === import46.DefaultValueAccessor) && (18 === requestNodeIndex))) { return this._DefaultValueAccessor_18_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (18 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_18_4; }
    if (((token === import47.FormControlName) && (18 === requestNodeIndex))) { return this._FormControlName_18_5; }
    if (((token === import50.NgControl) && (18 === requestNodeIndex))) { return this._NgControl_18_6; }
    if (((token === import45.NgControlStatus) && (18 === requestNodeIndex))) { return this._NgControlStatus_18_7; }
    if (((token === import46.DefaultValueAccessor) && (21 === requestNodeIndex))) { return this._DefaultValueAccessor_21_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (21 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_21_4; }
    if (((token === import47.FormControlName) && (21 === requestNodeIndex))) { return this._FormControlName_21_5; }
    if (((token === import50.NgControl) && (21 === requestNodeIndex))) { return this._NgControl_21_6; }
    if (((token === import45.NgControlStatus) && (21 === requestNodeIndex))) { return this._NgControlStatus_21_7; }
    if (((token === import46.DefaultValueAccessor) && (24 === requestNodeIndex))) { return this._DefaultValueAccessor_24_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (24 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_24_4; }
    if (((token === import47.FormControlName) && (24 === requestNodeIndex))) { return this._FormControlName_24_5; }
    if (((token === import50.NgControl) && (24 === requestNodeIndex))) { return this._NgControl_24_6; }
    if (((token === import45.NgControlStatus) && (24 === requestNodeIndex))) { return this._NgControlStatus_24_7; }
    if (((token === import46.DefaultValueAccessor) && (27 === requestNodeIndex))) { return this._DefaultValueAccessor_27_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (27 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_27_4; }
    if (((token === import47.FormControlName) && (27 === requestNodeIndex))) { return this._FormControlName_27_5; }
    if (((token === import50.NgControl) && (27 === requestNodeIndex))) { return this._NgControl_27_6; }
    if (((token === import45.NgControlStatus) && (27 === requestNodeIndex))) { return this._NgControlStatus_27_7; }
    if (((token === import46.DefaultValueAccessor) && (30 === requestNodeIndex))) { return this._DefaultValueAccessor_30_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (30 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_30_4; }
    if (((token === import47.FormControlName) && (30 === requestNodeIndex))) { return this._FormControlName_30_5; }
    if (((token === import50.NgControl) && (30 === requestNodeIndex))) { return this._NgControl_30_6; }
    if (((token === import45.NgControlStatus) && (30 === requestNodeIndex))) { return this._NgControlStatus_30_7; }
    if (((token === import46.DefaultValueAccessor) && (33 === requestNodeIndex))) { return this._DefaultValueAccessor_33_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (33 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_33_4; }
    if (((token === import47.FormControlName) && (33 === requestNodeIndex))) { return this._FormControlName_33_5; }
    if (((token === import50.NgControl) && (33 === requestNodeIndex))) { return this._NgControl_33_6; }
    if (((token === import45.NgControlStatus) && (33 === requestNodeIndex))) { return this._NgControlStatus_33_7; }
    if (((token === import46.DefaultValueAccessor) && (36 === requestNodeIndex))) { return this._DefaultValueAccessor_36_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (36 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_36_4; }
    if (((token === import47.FormControlName) && (36 === requestNodeIndex))) { return this._FormControlName_36_5; }
    if (((token === import50.NgControl) && (36 === requestNodeIndex))) { return this._NgControl_36_6; }
    if (((token === import45.NgControlStatus) && (36 === requestNodeIndex))) { return this._NgControlStatus_36_7; }
    if (((token === import46.DefaultValueAccessor) && (39 === requestNodeIndex))) { return this._DefaultValueAccessor_39_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (39 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_39_4; }
    if (((token === import47.FormControlName) && (39 === requestNodeIndex))) { return this._FormControlName_39_5; }
    if (((token === import50.NgControl) && (39 === requestNodeIndex))) { return this._NgControl_39_6; }
    if (((token === import45.NgControlStatus) && (39 === requestNodeIndex))) { return this._NgControlStatus_39_7; }
    if (((token === import46.DefaultValueAccessor) && (42 === requestNodeIndex))) { return this._DefaultValueAccessor_42_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (42 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_42_4; }
    if (((token === import47.FormControlName) && (42 === requestNodeIndex))) { return this._FormControlName_42_5; }
    if (((token === import50.NgControl) && (42 === requestNodeIndex))) { return this._NgControl_42_6; }
    if (((token === import45.NgControlStatus) && (42 === requestNodeIndex))) { return this._NgControlStatus_42_7; }
    if (((token === import46.DefaultValueAccessor) && (45 === requestNodeIndex))) { return this._DefaultValueAccessor_45_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (45 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_45_4; }
    if (((token === import47.FormControlName) && (45 === requestNodeIndex))) { return this._FormControlName_45_5; }
    if (((token === import50.NgControl) && (45 === requestNodeIndex))) { return this._NgControl_45_6; }
    if (((token === import45.NgControlStatus) && (45 === requestNodeIndex))) { return this._NgControlStatus_45_7; }
    if (((token === import46.DefaultValueAccessor) && (48 === requestNodeIndex))) { return this._DefaultValueAccessor_48_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (48 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_48_4; }
    if (((token === import47.FormControlName) && (48 === requestNodeIndex))) { return this._FormControlName_48_5; }
    if (((token === import50.NgControl) && (48 === requestNodeIndex))) { return this._NgControl_48_6; }
    if (((token === import45.NgControlStatus) && (48 === requestNodeIndex))) { return this._NgControlStatus_48_7; }
    if (((token === import46.DefaultValueAccessor) && (51 === requestNodeIndex))) { return this._DefaultValueAccessor_51_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (51 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_51_4; }
    if (((token === import47.FormControlName) && (51 === requestNodeIndex))) { return this._FormControlName_51_5; }
    if (((token === import50.NgControl) && (51 === requestNodeIndex))) { return this._NgControl_51_6; }
    if (((token === import45.NgControlStatus) && (51 === requestNodeIndex))) { return this._NgControlStatus_51_7; }
    if (((token === import46.DefaultValueAccessor) && (54 === requestNodeIndex))) { return this._DefaultValueAccessor_54_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (54 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_54_4; }
    if (((token === import47.FormControlName) && (54 === requestNodeIndex))) { return this._FormControlName_54_5; }
    if (((token === import50.NgControl) && (54 === requestNodeIndex))) { return this._NgControl_54_6; }
    if (((token === import45.NgControlStatus) && (54 === requestNodeIndex))) { return this._NgControlStatus_54_7; }
    if (((token === import46.DefaultValueAccessor) && (57 === requestNodeIndex))) { return this._DefaultValueAccessor_57_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (57 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_57_4; }
    if (((token === import47.FormControlName) && (57 === requestNodeIndex))) { return this._FormControlName_57_5; }
    if (((token === import50.NgControl) && (57 === requestNodeIndex))) { return this._NgControl_57_6; }
    if (((token === import45.NgControlStatus) && (57 === requestNodeIndex))) { return this._NgControlStatus_57_7; }
    if (((token === import46.DefaultValueAccessor) && (60 === requestNodeIndex))) { return this._DefaultValueAccessor_60_3; }
    if (((token === import49.NG_VALUE_ACCESSOR) && (60 === requestNodeIndex))) { return this._NG_VALUE_ACCESSOR_60_4; }
    if (((token === import47.FormControlName) && (60 === requestNodeIndex))) { return this._FormControlName_60_5; }
    if (((token === import50.NgControl) && (60 === requestNodeIndex))) { return this._NgControl_60_6; }
    if (((token === import45.NgControlStatus) && (60 === requestNodeIndex))) { return this._NgControlStatus_60_7; }
    if (((token === import44.FormGroupDirective) && ((1 <= requestNodeIndex) && (requestNodeIndex <= 62)))) { return this._FormGroupDirective_1_3; }
    if (((token === import51.ControlContainer) && ((1 <= requestNodeIndex) && (requestNodeIndex <= 62)))) { return this._ControlContainer_1_4; }
    if (((token === import45.NgControlStatusGroup) && ((1 <= requestNodeIndex) && (requestNodeIndex <= 62)))) { return this._NgControlStatusGroup_1_5; }
    return notFoundResult;
  }
}
export function viewFactory_AppComponent0(viewUtils:any,parentInjector:any,declarationEl:any):any {
  return new _View_AppComponent0(viewUtils,parentInjector,declarationEl);
}


//// [complexNarrowingWithAny.js]
// Repro from #10869
/**
 * This file is generated by the Angular 2 template compiler.
 * Do not edit.
 */
/* tslint:disable */
// import * as import0 from '@angular/core/src/linker/ng_module_factory';
// import * as import1 from '../../app';
// import * as import2 from '@angular/common/src/common_module';
// import * as import3 from '@angular/core/src/application_module';
// import * as import4 from '@angular/platform-browser/src/browser';
// import * as import5 from '@angular/forms/src/directives';
// import * as import6 from '@angular/forms/src/form_providers';
// import * as import7 from '@angular/common/src/localization';
// import * as import8 from '@angular/core/src/application_init';
// import * as import9 from '@angular/core/src/testability/testability';
// import * as import10 from '@angular/core/src/application_ref';
// import * as import11 from '@angular/core/src/linker/compiler';
// import * as import12 from '@angular/platform-browser/src/dom/events/hammer_gestures';
// import * as import13 from '@angular/platform-browser/src/dom/events/event_manager';
// import * as import14 from '@angular/platform-browser/src/dom/shared_styles_host';
// import * as import15 from '@angular/platform-browser/src/dom/dom_renderer';
// import * as import16 from '@angular/platform-browser/src/security/dom_sanitization_service';
// import * as import17 from '@angular/core/src/linker/view_utils';
// import * as import18 from '@angular/forms/src/form_builder';
// import * as import19 from '@angular/forms/src/directives/radio_control_value_accessor';
// import * as import20 from '@angular/core/src/di/injector';
// import * as import21 from '@angular/core/src/application_tokens';
// import * as import22 from '@angular/platform-browser/src/dom/events/dom_events';
// import * as import23 from '@angular/platform-browser/src/dom/events/key_events';
// import * as import24 from '@angular/core/src/zone/ng_zone';
// import * as import25 from '@angular/platform-browser/src/dom/debug/ng_probe';
// import * as import26 from '@angular/core/src/console';
// import * as import27 from '@angular/core/src/i18n/tokens';
// import * as import28 from '@angular/core/src/error_handler';
// import * as import29 from '@angular/platform-browser/src/dom/dom_tokens';
// import * as import30 from '@angular/platform-browser/src/dom/animation_driver';
// import * as import31 from '@angular/core/src/render/api';
// import * as import32 from '@angular/core/src/security';
// import * as import33 from '@angular/core/src/change_detection/differs/iterable_differs';
// import * as import34 from '@angular/core/src/change_detection/differs/keyvalue_differs';
// import * as import35 from '@angular/core/src/i18n/tokens';
// import * as import36 from '@angular/core/src/render/api';
// import * as import37 from '@angular/core/src/linker/view';
// import * as import38 from '@angular/core/src/linker/element';
// import * as import39 from '@angular/core/src/linker/view_utils';
// import * as import40 from '@angular/core/src/linker/view_type';
// import * as import41 from '@angular/core/src/change_detection/change_detection';
// import * as import42 from '@angular/core/src/metadata/view';
// import * as import43 from '@angular/core/src/linker/component_factory';
// import * as import44 from '@angular/forms/src/directives/reactive_directives/form_group_directive';
// import * as import45 from '@angular/forms/src/directives/ng_control_status';
// import * as import46 from '@angular/forms/src/directives/default_value_accessor';
// import * as import47 from '@angular/forms/src/directives/reactive_directives/form_control_name';
// import * as import48 from '@angular/core/src/linker/element_ref';
// import * as import49 from '@angular/forms/src/directives/control_value_accessor';
// import * as import50 from '@angular/forms/src/directives/ng_control';
// import * as import51 from '@angular/forms/src/directives/control_container';
//stubbed out imports
var import44;
(function (import44) {
    class FormGroupDirective {
        constructor(any) { }
    }
    import44.FormGroupDirective = FormGroupDirective;
})(import44 || (import44 = {}));
var import45;
(function (import45) {
    class NgControlStatus {
        constructor(any) { }
    }
    import45.NgControlStatus = NgControlStatus;
    class NgControlStatusGroup {
        constructor(any) { }
    }
    import45.NgControlStatusGroup = NgControlStatusGroup;
})(import45 || (import45 = {}));
var import46;
(function (import46) {
    class DefaultValueAccessor {
        constructor(any) { }
    }
    import46.DefaultValueAccessor = DefaultValueAccessor;
})(import46 || (import46 = {}));
var import47;
(function (import47) {
    class FormControlName {
        constructor(any) { }
    }
    import47.FormControlName = FormControlName;
})(import47 || (import47 = {}));
var import48;
(function (import48) {
    class FormControlName {
        constructor(any) { }
    }
    import48.FormControlName = FormControlName;
})(import48 || (import48 = {}));
//HERE BE DRAGONS
//Using a value here - 65+ seconds to typecheck
var import49;
(function (import49) {
    //real code uses an opaque token, using new String() to simulate.
    //export var NG_VALUE_ACCESSOR = new OpaqueToken('ngValueAccessor')
    import49.NG_VALUE_ACCESSOR = new String('foo');
})(import49 || (import49 = {}));
//using a class - < 1 sec typecheck
// namespace import49 {
// 	export class NG_VALUE_ACCESSOR {
// 		constructor(any){}
// 	}
// }
//END DRAGONS
var import50;
(function (import50) {
    class NgControl {
        constructor(any) { }
    }
    import50.NgControl = NgControl;
})(import50 || (import50 = {}));
var import51;
(function (import51) {
    class ControlContainer {
        constructor(any) { }
    }
    import51.ControlContainer = ControlContainer;
})(import51 || (import51 = {}));
class _View_AppComponent0 {
    constructor(viewUtils, parentInjector, declarationEl) {
    }
    injectorGetInternal(token, requestNodeIndex, notFoundResult) {
        if (((token === import46.DefaultValueAccessor) && (3 === requestNodeIndex))) {
            return this._DefaultValueAccessor_3_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (3 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_3_4;
        }
        if (((token === import47.FormControlName) && (3 === requestNodeIndex))) {
            return this._FormControlName_3_5;
        }
        if (((token === import50.NgControl) && (3 === requestNodeIndex))) {
            return this._NgControl_3_6;
        }
        if (((token === import45.NgControlStatus) && (3 === requestNodeIndex))) {
            return this._NgControlStatus_3_7;
        }
        if (((token === import46.DefaultValueAccessor) && (6 === requestNodeIndex))) {
            return this._DefaultValueAccessor_6_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (6 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_6_4;
        }
        if (((token === import47.FormControlName) && (6 === requestNodeIndex))) {
            return this._FormControlName_6_5;
        }
        if (((token === import50.NgControl) && (6 === requestNodeIndex))) {
            return this._NgControl_6_6;
        }
        if (((token === import45.NgControlStatus) && (6 === requestNodeIndex))) {
            return this._NgControlStatus_6_7;
        }
        if (((token === import46.DefaultValueAccessor) && (9 === requestNodeIndex))) {
            return this._DefaultValueAccessor_9_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (9 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_9_4;
        }
        if (((token === import47.FormControlName) && (9 === requestNodeIndex))) {
            return this._FormControlName_9_5;
        }
        if (((token === import50.NgControl) && (9 === requestNodeIndex))) {
            return this._NgControl_9_6;
        }
        if (((token === import45.NgControlStatus) && (9 === requestNodeIndex))) {
            return this._NgControlStatus_9_7;
        }
        if (((token === import46.DefaultValueAccessor) && (12 === requestNodeIndex))) {
            return this._DefaultValueAccessor_12_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (12 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_12_4;
        }
        if (((token === import47.FormControlName) && (12 === requestNodeIndex))) {
            return this._FormControlName_12_5;
        }
        if (((token === import50.NgControl) && (12 === requestNodeIndex))) {
            return this._NgControl_12_6;
        }
        if (((token === import45.NgControlStatus) && (12 === requestNodeIndex))) {
            return this._NgControlStatus_12_7;
        }
        if (((token === import46.DefaultValueAccessor) && (15 === requestNodeIndex))) {
            return this._DefaultValueAccessor_15_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (15 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_15_4;
        }
        if (((token === import47.FormControlName) && (15 === requestNodeIndex))) {
            return this._FormControlName_15_5;
        }
        if (((token === import50.NgControl) && (15 === requestNodeIndex))) {
            return this._NgControl_15_6;
        }
        if (((token === import45.NgControlStatus) && (15 === requestNodeIndex))) {
            return this._NgControlStatus_15_7;
        }
        if (((token === import46.DefaultValueAccessor) && (18 === requestNodeIndex))) {
            return this._DefaultValueAccessor_18_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (18 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_18_4;
        }
        if (((token === import47.FormControlName) && (18 === requestNodeIndex))) {
            return this._FormControlName_18_5;
        }
        if (((token === import50.NgControl) && (18 === requestNodeIndex))) {
            return this._NgControl_18_6;
        }
        if (((token === import45.NgControlStatus) && (18 === requestNodeIndex))) {
            return this._NgControlStatus_18_7;
        }
        if (((token === import46.DefaultValueAccessor) && (21 === requestNodeIndex))) {
            return this._DefaultValueAccessor_21_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (21 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_21_4;
        }
        if (((token === import47.FormControlName) && (21 === requestNodeIndex))) {
            return this._FormControlName_21_5;
        }
        if (((token === import50.NgControl) && (21 === requestNodeIndex))) {
            return this._NgControl_21_6;
        }
        if (((token === import45.NgControlStatus) && (21 === requestNodeIndex))) {
            return this._NgControlStatus_21_7;
        }
        if (((token === import46.DefaultValueAccessor) && (24 === requestNodeIndex))) {
            return this._DefaultValueAccessor_24_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (24 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_24_4;
        }
        if (((token === import47.FormControlName) && (24 === requestNodeIndex))) {
            return this._FormControlName_24_5;
        }
        if (((token === import50.NgControl) && (24 === requestNodeIndex))) {
            return this._NgControl_24_6;
        }
        if (((token === import45.NgControlStatus) && (24 === requestNodeIndex))) {
            return this._NgControlStatus_24_7;
        }
        if (((token === import46.DefaultValueAccessor) && (27 === requestNodeIndex))) {
            return this._DefaultValueAccessor_27_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (27 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_27_4;
        }
        if (((token === import47.FormControlName) && (27 === requestNodeIndex))) {
            return this._FormControlName_27_5;
        }
        if (((token === import50.NgControl) && (27 === requestNodeIndex))) {
            return this._NgControl_27_6;
        }
        if (((token === import45.NgControlStatus) && (27 === requestNodeIndex))) {
            return this._NgControlStatus_27_7;
        }
        if (((token === import46.DefaultValueAccessor) && (30 === requestNodeIndex))) {
            return this._DefaultValueAccessor_30_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (30 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_30_4;
        }
        if (((token === import47.FormControlName) && (30 === requestNodeIndex))) {
            return this._FormControlName_30_5;
        }
        if (((token === import50.NgControl) && (30 === requestNodeIndex))) {
            return this._NgControl_30_6;
        }
        if (((token === import45.NgControlStatus) && (30 === requestNodeIndex))) {
            return this._NgControlStatus_30_7;
        }
        if (((token === import46.DefaultValueAccessor) && (33 === requestNodeIndex))) {
            return this._DefaultValueAccessor_33_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (33 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_33_4;
        }
        if (((token === import47.FormControlName) && (33 === requestNodeIndex))) {
            return this._FormControlName_33_5;
        }
        if (((token === import50.NgControl) && (33 === requestNodeIndex))) {
            return this._NgControl_33_6;
        }
        if (((token === import45.NgControlStatus) && (33 === requestNodeIndex))) {
            return this._NgControlStatus_33_7;
        }
        if (((token === import46.DefaultValueAccessor) && (36 === requestNodeIndex))) {
            return this._DefaultValueAccessor_36_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (36 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_36_4;
        }
        if (((token === import47.FormControlName) && (36 === requestNodeIndex))) {
            return this._FormControlName_36_5;
        }
        if (((token === import50.NgControl) && (36 === requestNodeIndex))) {
            return this._NgControl_36_6;
        }
        if (((token === import45.NgControlStatus) && (36 === requestNodeIndex))) {
            return this._NgControlStatus_36_7;
        }
        if (((token === import46.DefaultValueAccessor) && (39 === requestNodeIndex))) {
            return this._DefaultValueAccessor_39_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (39 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_39_4;
        }
        if (((token === import47.FormControlName) && (39 === requestNodeIndex))) {
            return this._FormControlName_39_5;
        }
        if (((token === import50.NgControl) && (39 === requestNodeIndex))) {
            return this._NgControl_39_6;
        }
        if (((token === import45.NgControlStatus) && (39 === requestNodeIndex))) {
            return this._NgControlStatus_39_7;
        }
        if (((token === import46.DefaultValueAccessor) && (42 === requestNodeIndex))) {
            return this._DefaultValueAccessor_42_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (42 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_42_4;
        }
        if (((token === import47.FormControlName) && (42 === requestNodeIndex))) {
            return this._FormControlName_42_5;
        }
        if (((token === import50.NgControl) && (42 === requestNodeIndex))) {
            return this._NgControl_42_6;
        }
        if (((token === import45.NgControlStatus) && (42 === requestNodeIndex))) {
            return this._NgControlStatus_42_7;
        }
        if (((token === import46.DefaultValueAccessor) && (45 === requestNodeIndex))) {
            return this._DefaultValueAccessor_45_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (45 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_45_4;
        }
        if (((token === import47.FormControlName) && (45 === requestNodeIndex))) {
            return this._FormControlName_45_5;
        }
        if (((token === import50.NgControl) && (45 === requestNodeIndex))) {
            return this._NgControl_45_6;
        }
        if (((token === import45.NgControlStatus) && (45 === requestNodeIndex))) {
            return this._NgControlStatus_45_7;
        }
        if (((token === import46.DefaultValueAccessor) && (48 === requestNodeIndex))) {
            return this._DefaultValueAccessor_48_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (48 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_48_4;
        }
        if (((token === import47.FormControlName) && (48 === requestNodeIndex))) {
            return this._FormControlName_48_5;
        }
        if (((token === import50.NgControl) && (48 === requestNodeIndex))) {
            return this._NgControl_48_6;
        }
        if (((token === import45.NgControlStatus) && (48 === requestNodeIndex))) {
            return this._NgControlStatus_48_7;
        }
        if (((token === import46.DefaultValueAccessor) && (51 === requestNodeIndex))) {
            return this._DefaultValueAccessor_51_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (51 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_51_4;
        }
        if (((token === import47.FormControlName) && (51 === requestNodeIndex))) {
            return this._FormControlName_51_5;
        }
        if (((token === import50.NgControl) && (51 === requestNodeIndex))) {
            return this._NgControl_51_6;
        }
        if (((token === import45.NgControlStatus) && (51 === requestNodeIndex))) {
            return this._NgControlStatus_51_7;
        }
        if (((token === import46.DefaultValueAccessor) && (54 === requestNodeIndex))) {
            return this._DefaultValueAccessor_54_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (54 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_54_4;
        }
        if (((token === import47.FormControlName) && (54 === requestNodeIndex))) {
            return this._FormControlName_54_5;
        }
        if (((token === import50.NgControl) && (54 === requestNodeIndex))) {
            return this._NgControl_54_6;
        }
        if (((token === import45.NgControlStatus) && (54 === requestNodeIndex))) {
            return this._NgControlStatus_54_7;
        }
        if (((token === import46.DefaultValueAccessor) && (57 === requestNodeIndex))) {
            return this._DefaultValueAccessor_57_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (57 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_57_4;
        }
        if (((token === import47.FormControlName) && (57 === requestNodeIndex))) {
            return this._FormControlName_57_5;
        }
        if (((token === import50.NgControl) && (57 === requestNodeIndex))) {
            return this._NgControl_57_6;
        }
        if (((token === import45.NgControlStatus) && (57 === requestNodeIndex))) {
            return this._NgControlStatus_57_7;
        }
        if (((token === import46.DefaultValueAccessor) && (60 === requestNodeIndex))) {
            return this._DefaultValueAccessor_60_3;
        }
        if (((token === import49.NG_VALUE_ACCESSOR) && (60 === requestNodeIndex))) {
            return this._NG_VALUE_ACCESSOR_60_4;
        }
        if (((token === import47.FormControlName) && (60 === requestNodeIndex))) {
            return this._FormControlName_60_5;
        }
        if (((token === import50.NgControl) && (60 === requestNodeIndex))) {
            return this._NgControl_60_6;
        }
        if (((token === import45.NgControlStatus) && (60 === requestNodeIndex))) {
            return this._NgControlStatus_60_7;
        }
        if (((token === import44.FormGroupDirective) && ((1 <= requestNodeIndex) && (requestNodeIndex <= 62)))) {
            return this._FormGroupDirective_1_3;
        }
        if (((token === import51.ControlContainer) && ((1 <= requestNodeIndex) && (requestNodeIndex <= 62)))) {
            return this._ControlContainer_1_4;
        }
        if (((token === import45.NgControlStatusGroup) && ((1 <= requestNodeIndex) && (requestNodeIndex <= 62)))) {
            return this._NgControlStatusGroup_1_5;
        }
        return notFoundResult;
    }
}
export function viewFactory_AppComponent0(viewUtils, parentInjector, declarationEl) {
    return new _View_AppComponent0(viewUtils, parentInjector, declarationEl);
}
