0.ts --module
CompilerOptions::
{}
WatchOptions::

FileNames::
0.ts
Errors::
error TS6044: Compiler option 'module' expects an argument.
error TS6046: Argument for '--module' option must be: 'commonjs', 'es6', 'es2015', 'es2020', 'es2022', 'esnext', 'node16', 'node18', 'node20', 'nodenext', 'preserve'.
