//// [tests/cases/conformance/async/es5/asyncArrowFunction/asyncArrowFunction4_es5.ts] ////

//// [asyncArrowFunction4_es5.ts]
var await = () => {
}

//// [asyncArrowFunction4_es5.js]
"use strict";
var await = function () {
};
