/* Generated file to emulate the ts namespace. */

export * from "../../compiler/_namespaces/ts.js";
export * from "../../jsTyping/_namespaces/ts.js";
export * from "../types.js";
export * from "../utilities.js";
export * from "../exportInfoMap.js";
export * from "../classifier.js";
export * from "../documentHighlights.js";
export * from "../documentRegistry.js";
export * from "../getEditsForFileRename.js";
export * from "../patternMatcher.js";
export * from "../preProcess.js";
export * from "../sourcemaps.js";
export * from "../suggestionDiagnostics.js";
export * from "../transpile.js";
export * from "../services.js";
export * from "../transform.js";
import * as BreakpointResolver from "./ts.BreakpointResolver.js";
export { BreakpointResolver };
import * as CallHierarchy from "./ts.CallHierarchy.js";
export { CallHierarchy };
import * as classifier from "./ts.classifier.js";
export { classifier };
import * as codefix from "./ts.codefix.js";
export { codefix };
import * as Completions from "./ts.Completions.js";
export { Completions };
import * as FindAllReferences from "./ts.FindAllReferences.js";
export { FindAllReferences };
import * as GoToDefinition from "./ts.GoToDefinition.js";
export { GoToDefinition };
import * as InlayHints from "./ts.InlayHints.js";
export { InlayHints };
import * as JsDoc from "./ts.JsDoc.js";
import * as MapCode from "./ts.MapCode.js";
export { MapCode };
export { JsDoc };
import * as NavigateTo from "./ts.NavigateTo.js";
export { NavigateTo };
import * as NavigationBar from "./ts.NavigationBar.js";
export { NavigationBar };
import * as OrganizeImports from "./ts.OrganizeImports.js";
export { OrganizeImports };
import * as OutliningElementsCollector from "./ts.OutliningElementsCollector.js";
export { OutliningElementsCollector };
import * as refactor from "./ts.refactor.js";
export { refactor };
import * as Rename from "./ts.Rename.js";
export { Rename };
import * as SignatureHelp from "./ts.SignatureHelp.js";
export { SignatureHelp };
import * as SmartSelectionRange from "./ts.SmartSelectionRange.js";
export { SmartSelectionRange };
import * as SymbolDisplay from "./ts.SymbolDisplay.js";
export { SymbolDisplay };
import * as textChanges from "./ts.textChanges.js";
export { textChanges };
import * as formatting from "./ts.formatting.js";
export { formatting };
import * as PreparePasteEdits from "./ts.preparePasteEdits.js";
export { PreparePasteEdits };
import * as pasteEdits from "./ts.PasteEdits.js";
export { pasteEdits };
