//// [tests/cases/compiler/catchClauseWithInitializer1.ts] ////

//// [catchClauseWithInitializer1.ts]
try {
}
catch (e = 1) {
}

//// [catchClauseWithInitializer1.js]
"use strict";
try {
}
catch (e = 1) {
}
