//// [tests/cases/conformance/classes/classStaticBlock/classStaticBlock15.ts] ////

//// [classStaticBlock15.ts]
var _C__1;

class C {
  static #_1 = 1;
  static #_3 = 3;
  static #_5 = 5;

  static {}
  static {}
  static {}
  static {}
  static {}
  static {}
}

console.log(_C__1)


//// [classStaticBlock15.js]
"use strict";
var _a, _C__1_1, _C__3, _C__5;
var _C__1;
class C {
}
_a = C;
_C__1_1 = { value: 1 };
_C__3 = { value: 3 };
_C__5 = { value: 5 };
(() => {
})();
(() => {
})();
(() => {
})();
(() => {
})();
(() => {
})();
(() => {
})();
console.log(_C__1);
