//// [tests/cases/conformance/es6/functionPropertyAssignments/FunctionPropertyAssignments5_es6.ts] ////

//// [FunctionPropertyAssignments5_es6.ts]
var v = { *[foo()]() { } }

//// [FunctionPropertyAssignments5_es6.js]
"use strict";
var v = { *[foo()]() { } };
