//// [tests/cases/conformance/async/es6/asyncArrowFunction/arrowFunctionWithParameterNameAsync_es6.ts] ////

//// [arrowFunctionWithParameterNameAsync_es6.ts]
const x = async => async;

//// [arrowFunctionWithParameterNameAsync_es6.js]
"use strict";
const x = async => async;
