//// [tests/cases/conformance/async/es2017/asyncArrowFunction/asyncArrowFunction3_es2017.ts] ////

//// [asyncArrowFunction3_es2017.ts]
function f(await = await) {
}

//// [asyncArrowFunction3_es2017.js]
"use strict";
function f(await = await) {
}
