//// [tests/cases/compiler/commentOnArrayElement8.ts] ////

//// [commentOnArrayElement8.ts]
const array = [
    1, /* comment */
];


//// [commentOnArrayElement8.js]
"use strict";
const array = [
    1, /* comment */
];
