//// [tests/cases/conformance/es6/destructuring/emptyAssignmentPatterns01_ES6.ts] ////

//// [emptyAssignmentPatterns01_ES6.ts]
var a: any;

({} = a);
([] = a);

//// [emptyAssignmentPatterns01_ES6.js]
"use strict";
var a;
({} = a);
([] = a);


//// [emptyAssignmentPatterns01_ES6.d.ts]
declare var a: any;
