//// [tests/cases/compiler/commentOnArrayElement9.ts] ////

//// [commentOnArrayElement9.ts]
const array = [
    /* element 1 */ 1, /* end of element 1 */
];


//// [commentOnArrayElement9.js]
"use strict";
const array = [
    /* element 1 */ 1, /* end of element 1 */
];
