//// [tests/cases/compiler/defaultOfAnyInStrictNullChecks.ts] ////

//// [defaultOfAnyInStrictNullChecks.ts]
// Regression test for #8295

function foo() {
    try {
    }
    catch (e) {
        let s = e.message; 
    }
}


//// [defaultOfAnyInStrictNullChecks.js]
"use strict";
// Regression test for #8295
function foo() {
    try {
    }
    catch (e) {
        let s = e.message;
    }
}
