--tsBuildInfoFile build.tsbuildinfo tests
buildOptions::
{}
WatchOptions::

Projects::
build.tsbuildinfo,tests
Errors::
error TS5094: Compiler option '--tsBuildInfoFile' may not be used with '--build'.
