//// [tests/cases/conformance/statements/for-ofStatements/ES5For-of29.ts] ////

//// [ES5For-of29.ts]
for (const {x: a = 0, y: b = 1} of [2, 3]) {
    a;
    b;
}

//// [ES5For-of29.js]
"use strict";
for (var _i = 0, _a = [2, 3]; _i < _a.length; _i++) {
    var _b = _a[_i], _c = _b.x, a = _c === void 0 ? 0 : _c, _d = _b.y, b = _d === void 0 ? 1 : _d;
    a;
    b;
}
