//// [tests/cases/conformance/constEnums/constEnum4.ts] ////

//// [constEnum4.ts]
if (1)
    const enum A { }
else if (2)
    const enum B { }
else
    const enum C { }


//// [constEnum4.js]
"use strict";
if (1)
    ;
else if (2)
    ;
else
    ;
