//// [tests/cases/conformance/classes/classStaticBlock/classStaticBlock5.ts] ////

//// [classStaticBlock5.ts]
class B {
    static a = 1;
    static b = 2;
}

class C extends B {
    static b = 3;
    static c = super.a

    static {
        this.b;
        super.b;
        super.a;
    }
}


//// [classStaticBlock5.js]
"use strict";
class B {
    static a = 1;
    static b = 2;
}
class C extends B {
    static b = 3;
    static c = super.a;
    static {
        this.b;
        super.b;
        super.a;
    }
}
