//// [tests/cases/conformance/async/es5/asyncGetter_es5.ts] ////

//// [asyncGetter_es5.ts]
class C {
  async get foo() {
  }
}

//// [asyncGetter_es5.js]
"use strict";
class C {
    get foo() {
    }
}
