//// [tests/cases/conformance/parser/ecmascript5/ErrorRecovery/ArrowFunctions/ArrowFunction1.ts] ////

//// [ArrowFunction1.ts]
var v = (a: ) => {
   
};

//// [ArrowFunction1.js]
"use strict";
var v = (a) => {
};
