//// [tests/cases/compiler/declarationEmitDefaultExport4.ts] ////

//// [declarationEmitDefaultExport4.ts]
export default function () {
    return 1;
}

//// [declarationEmitDefaultExport4.js]
export default function () {
    return 1;
}


//// [declarationEmitDefaultExport4.d.ts]
export default function (): number;
